
import google.protobuf
from packaging.version import Version
if Version(google.protobuf.__version__).major >= 5:
  # -*- coding: utf-8 -*-
  # Generated by the protocol buffer compiler.  DO NOT EDIT!
  # source: unity_catalog_oss_messages.proto
  # Protobuf Python Version: 5.26.0
  """Generated protocol buffer code."""
  from google.protobuf import descriptor as _descriptor
  from google.protobuf import descriptor_pool as _descriptor_pool
  from google.protobuf import symbol_database as _symbol_database
  from google.protobuf.internal import builder as _builder
  # @@protoc_insertion_point(imports)

  _sym_db = _symbol_database.Default()


  from . import databricks_pb2 as databricks__pb2
  from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2


  DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n unity_catalog_oss_messages.proto\x12\x13mlflow.unitycatalog\x1a\x10\x64\x61tabricks.proto\x1a\x15scalapb/scalapb.proto\"\xfd\x01\n\x13RegisteredModelInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0c\x63\x61talog_name\x18\x02 \x01(\t\x12\x13\n\x0bschema_name\x18\x03 \x01(\t\x12\x0f\n\x07\x63omment\x18\x05 \x01(\t\x12\x18\n\x10storage_location\x18\x06 \x01(\t\x12\x11\n\tfull_name\x18\t \x01(\t\x12\x12\n\ncreated_at\x18\x0b \x01(\x03\x12\x12\n\ncreated_by\x18\x0c \x01(\t\x12\x12\n\nupdated_at\x18\r \x01(\x03\x12\x12\n\nupdated_by\x18\x0e \x01(\t\x12\n\n\x02id\x18\x12 \x01(\t\x12\x13\n\x0b\x62rowse_only\x18\x15 \x01(\x08\"\xd0\x01\n\x15\x43reateRegisteredModel\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0c\x63\x61talog_name\x18\x02 \x01(\t\x12\x13\n\x0bschema_name\x18\x03 \x01(\t\x12\x0f\n\x07\x63omment\x18\x05 \x01(\t\x12\x18\n\x10storage_location\x18\x06 \x01(\t\x1aS\n\x08Response\x12G\n\x15registered_model_info\x18\x01 \x01(\x0b\x32(.mlflow.unitycatalog.RegisteredModelInfo\"E\n\x15\x44\x65leteRegisteredModel\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\r\n\x05\x66orce\x18\x02 \x01(\x08\x1a\n\n\x08Response\"\x82\x01\n\x12GetRegisteredModel\x12\x17\n\tfull_name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1aS\n\x08Response\x12G\n\x15registered_model_info\x18\x01 \x01(\x0b\x32(.mlflow.unitycatalog.RegisteredModelInfo\"\xa2\x01\n\x15UpdateRegisteredModel\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x10\n\x08new_name\x18\x03 \x01(\t\x12\x0f\n\x07\x63omment\x18\x02 \x01(\t\x1aS\n\x08Response\x12G\n\x15registered_model_info\x18\x01 \x01(\x0b\x32(.mlflow.unitycatalog.RegisteredModelInfo\"\xd4\x01\n\x14ListRegisteredModels\x12\x14\n\x0c\x63\x61talog_name\x18\x01 \x01(\t\x12\x13\n\x0bschema_name\x18\x02 \x01(\t\x12\x13\n\x0bmax_results\x18\x05 \x01(\x03\x12\x12\n\npage_token\x18\x06 \x01(\t\x1ah\n\x08Response\x12\x43\n\x11registered_models\x18\x01 \x03(\x0b\x32(.mlflow.unitycatalog.RegisteredModelInfo\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"\xc2\x02\n\x10ModelVersionInfo\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x14\n\x0c\x63\x61talog_name\x18\x02 \x01(\t\x12\x13\n\x0bschema_name\x18\x03 \x01(\t\x12\x0e\n\x06source\x18\x05 \x01(\t\x12\x0f\n\x07\x63omment\x18\x04 \x01(\t\x12\x0e\n\x06run_id\x18\x06 \x01(\t\x12\x37\n\x06status\x18\x08 \x01(\x0e\x32\'.mlflow.unitycatalog.ModelVersionStatus\x12\x0f\n\x07version\x18\t \x01(\x03\x12\x18\n\x10storage_location\x18\n \x01(\t\x12\x12\n\ncreated_at\x18\x0c \x01(\x03\x12\x12\n\ncreated_by\x18\r \x01(\t\x12\x12\n\nupdated_at\x18\x0e \x01(\x03\x12\x12\n\nupdated_by\x18\x0f \x01(\t\x12\n\n\x02id\x18\x10 \x01(\t\"\xd3\x01\n\x12\x43reateModelVersion\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x14\n\x0c\x63\x61talog_name\x18\x02 \x01(\t\x12\x13\n\x0bschema_name\x18\x03 \x01(\t\x12\x0e\n\x06source\x18\x04 \x01(\t\x12\x0e\n\x06run_id\x18\x05 \x01(\t\x12\x0f\n\x07\x63omment\x18\x06 \x01(\t\x1aM\n\x08Response\x12\x41\n\x12model_version_info\x18\x01 \x01(\x0b\x32%.mlflow.unitycatalog.ModelVersionInfo\"D\n\x12\x44\x65leteModelVersion\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x03\x1a\n\n\x08Response\"\x89\x01\n\x14\x46inalizeModelVersion\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x03\x1aM\n\x08Response\x12\x41\n\x12model_version_info\x18\x01 \x01(\x0b\x32%.mlflow.unitycatalog.ModelVersionInfo\"\x84\x01\n\x0fGetModelVersion\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x03\x1aM\n\x08Response\x12\x41\n\x12model_version_info\x18\x01 \x01(\x0b\x32%.mlflow.unitycatalog.ModelVersionInfo\"\x98\x01\n\x12UpdateModelVersion\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x03\x12\x0f\n\x07\x63omment\x18\x03 \x01(\t\x1aM\n\x08Response\x12\x41\n\x12model_version_info\x18\x01 \x01(\x0b\x32%.mlflow.unitycatalog.ModelVersionInfo\"\xb3\x01\n\x11ListModelVersions\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x13\n\x0bmax_results\x18\x04 \x01(\x03\x12\x12\n\npage_token\x18\x05 \x01(\t\x1a\x62\n\x08Response\x12=\n\x0emodel_versions\x18\x01 \x03(\x0b\x32%.mlflow.unitycatalog.ModelVersionInfo\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"\xff\x01\n\x14TemporaryCredentials\x12\x41\n\x14\x61ws_temp_credentials\x18\x02 \x01(\x0b\x32#.mlflow.unitycatalog.AwsCredentials\x12N\n\x19\x61zure_user_delegation_sas\x18\x03 \x01(\x0b\x32+.mlflow.unitycatalog.AzureUserDelegationSAS\x12;\n\x0fgcp_oauth_token\x18\x04 \x01(\x0b\x32\".mlflow.unitycatalog.GcpOauthToken\x12\x17\n\x0f\x65xpiration_time\x18\x01 \x01(\x03\"Y\n\x0e\x41wsCredentials\x12\x15\n\raccess_key_id\x18\x01 \x01(\t\x12\x19\n\x11secret_access_key\x18\x02 \x01(\t\x12\x15\n\rsession_token\x18\x03 \x01(\t\"+\n\x16\x41zureUserDelegationSAS\x12\x11\n\tsas_token\x18\x01 \x01(\t\"$\n\rGcpOauthToken\x12\x13\n\x0boauth_token\x18\x01 \x01(\t\"\x84\x02\n\'GenerateTemporaryModelVersionCredential\x12\x14\n\x0c\x63\x61talog_name\x18\x01 \x01(\t\x12\x13\n\x0bschema_name\x18\x02 \x01(\t\x12\x12\n\nmodel_name\x18\x03 \x01(\t\x12\x0f\n\x07version\x18\x04 \x01(\x03\x12=\n\toperation\x18\x05 \x01(\x0e\x32*.mlflow.unitycatalog.ModelVersionOperation\x1aJ\n\x08Response\x12>\n\x0b\x63redentials\x18\x01 \x01(\x0b\x32).mlflow.unitycatalog.TemporaryCredentials*t\n\x12ModelVersionStatus\x12 \n\x1cMODEL_VERSION_STATUS_UNKNOWN\x10\x00\x12\x18\n\x14PENDING_REGISTRATION\x10\x01\x12\x17\n\x13\x46\x41ILED_REGISTRATION\x10\x02\x12\t\n\x05READY\x10\x03*r\n\x15ModelVersionOperation\x12#\n\x1fUNKNOWN_MODEL_VERSION_OPERATION\x10\x00\x12\x16\n\x12READ_MODEL_VERSION\x10\x01\x12\x1c\n\x18READ_WRITE_MODEL_VERSION\x10\x02\x42\x31\n\'com.databricks.api.proto.managedcatalog\xa0\x01\x01\xe2?\x02\x10\x01')

  _globals = globals()
  _builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
  _builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'unity_catalog_oss_messages_pb2', _globals)
  if not _descriptor._USE_C_DESCRIPTORS:
    _globals['DESCRIPTOR']._loaded_options = None
    _globals['DESCRIPTOR']._serialized_options = b'\n\'com.databricks.api.proto.managedcatalog\240\001\001\342?\002\020\001'
    _globals['_GETREGISTEREDMODEL'].fields_by_name['full_name']._loaded_options = None
    _globals['_GETREGISTEREDMODEL'].fields_by_name['full_name']._serialized_options = b'\370\206\031\001'
    _globals['_MODELVERSIONSTATUS']._serialized_start=3065
    _globals['_MODELVERSIONSTATUS']._serialized_end=3181
    _globals['_MODELVERSIONOPERATION']._serialized_start=3183
    _globals['_MODELVERSIONOPERATION']._serialized_end=3297
    _globals['_REGISTEREDMODELINFO']._serialized_start=99
    _globals['_REGISTEREDMODELINFO']._serialized_end=352
    _globals['_CREATEREGISTEREDMODEL']._serialized_start=355
    _globals['_CREATEREGISTEREDMODEL']._serialized_end=563
    _globals['_CREATEREGISTEREDMODEL_RESPONSE']._serialized_start=480
    _globals['_CREATEREGISTEREDMODEL_RESPONSE']._serialized_end=563
    _globals['_DELETEREGISTEREDMODEL']._serialized_start=565
    _globals['_DELETEREGISTEREDMODEL']._serialized_end=634
    _globals['_DELETEREGISTEREDMODEL_RESPONSE']._serialized_start=480
    _globals['_DELETEREGISTEREDMODEL_RESPONSE']._serialized_end=490
    _globals['_GETREGISTEREDMODEL']._serialized_start=637
    _globals['_GETREGISTEREDMODEL']._serialized_end=767
    _globals['_GETREGISTEREDMODEL_RESPONSE']._serialized_start=480
    _globals['_GETREGISTEREDMODEL_RESPONSE']._serialized_end=563
    _globals['_UPDATEREGISTEREDMODEL']._serialized_start=770
    _globals['_UPDATEREGISTEREDMODEL']._serialized_end=932
    _globals['_UPDATEREGISTEREDMODEL_RESPONSE']._serialized_start=480
    _globals['_UPDATEREGISTEREDMODEL_RESPONSE']._serialized_end=563
    _globals['_LISTREGISTEREDMODELS']._serialized_start=935
    _globals['_LISTREGISTEREDMODELS']._serialized_end=1147
    _globals['_LISTREGISTEREDMODELS_RESPONSE']._serialized_start=1043
    _globals['_LISTREGISTEREDMODELS_RESPONSE']._serialized_end=1147
    _globals['_MODELVERSIONINFO']._serialized_start=1150
    _globals['_MODELVERSIONINFO']._serialized_end=1472
    _globals['_CREATEMODELVERSION']._serialized_start=1475
    _globals['_CREATEMODELVERSION']._serialized_end=1686
    _globals['_CREATEMODELVERSION_RESPONSE']._serialized_start=1609
    _globals['_CREATEMODELVERSION_RESPONSE']._serialized_end=1686
    _globals['_DELETEMODELVERSION']._serialized_start=1688
    _globals['_DELETEMODELVERSION']._serialized_end=1756
    _globals['_DELETEMODELVERSION_RESPONSE']._serialized_start=480
    _globals['_DELETEMODELVERSION_RESPONSE']._serialized_end=490
    _globals['_FINALIZEMODELVERSION']._serialized_start=1759
    _globals['_FINALIZEMODELVERSION']._serialized_end=1896
    _globals['_FINALIZEMODELVERSION_RESPONSE']._serialized_start=1609
    _globals['_FINALIZEMODELVERSION_RESPONSE']._serialized_end=1686
    _globals['_GETMODELVERSION']._serialized_start=1899
    _globals['_GETMODELVERSION']._serialized_end=2031
    _globals['_GETMODELVERSION_RESPONSE']._serialized_start=1609
    _globals['_GETMODELVERSION_RESPONSE']._serialized_end=1686
    _globals['_UPDATEMODELVERSION']._serialized_start=2034
    _globals['_UPDATEMODELVERSION']._serialized_end=2186
    _globals['_UPDATEMODELVERSION_RESPONSE']._serialized_start=1609
    _globals['_UPDATEMODELVERSION_RESPONSE']._serialized_end=1686
    _globals['_LISTMODELVERSIONS']._serialized_start=2189
    _globals['_LISTMODELVERSIONS']._serialized_end=2368
    _globals['_LISTMODELVERSIONS_RESPONSE']._serialized_start=2270
    _globals['_LISTMODELVERSIONS_RESPONSE']._serialized_end=2368
    _globals['_TEMPORARYCREDENTIALS']._serialized_start=2371
    _globals['_TEMPORARYCREDENTIALS']._serialized_end=2626
    _globals['_AWSCREDENTIALS']._serialized_start=2628
    _globals['_AWSCREDENTIALS']._serialized_end=2717
    _globals['_AZUREUSERDELEGATIONSAS']._serialized_start=2719
    _globals['_AZUREUSERDELEGATIONSAS']._serialized_end=2762
    _globals['_GCPOAUTHTOKEN']._serialized_start=2764
    _globals['_GCPOAUTHTOKEN']._serialized_end=2800
    _globals['_GENERATETEMPORARYMODELVERSIONCREDENTIAL']._serialized_start=2803
    _globals['_GENERATETEMPORARYMODELVERSIONCREDENTIAL']._serialized_end=3063
    _globals['_GENERATETEMPORARYMODELVERSIONCREDENTIAL_RESPONSE']._serialized_start=2989
    _globals['_GENERATETEMPORARYMODELVERSIONCREDENTIAL_RESPONSE']._serialized_end=3063
  # @@protoc_insertion_point(module_scope)

else:
  # -*- coding: utf-8 -*-
  # Generated by the protocol buffer compiler.  DO NOT EDIT!
  # source: unity_catalog_oss_messages.proto
  """Generated protocol buffer code."""
  from google.protobuf.internal import enum_type_wrapper
  from google.protobuf import descriptor as _descriptor
  from google.protobuf import descriptor_pool as _descriptor_pool
  from google.protobuf import message as _message
  from google.protobuf import reflection as _reflection
  from google.protobuf import symbol_database as _symbol_database
  # @@protoc_insertion_point(imports)

  _sym_db = _symbol_database.Default()


  from . import databricks_pb2 as databricks__pb2
  from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2


  DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n unity_catalog_oss_messages.proto\x12\x13mlflow.unitycatalog\x1a\x10\x64\x61tabricks.proto\x1a\x15scalapb/scalapb.proto\"\xfd\x01\n\x13RegisteredModelInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0c\x63\x61talog_name\x18\x02 \x01(\t\x12\x13\n\x0bschema_name\x18\x03 \x01(\t\x12\x0f\n\x07\x63omment\x18\x05 \x01(\t\x12\x18\n\x10storage_location\x18\x06 \x01(\t\x12\x11\n\tfull_name\x18\t \x01(\t\x12\x12\n\ncreated_at\x18\x0b \x01(\x03\x12\x12\n\ncreated_by\x18\x0c \x01(\t\x12\x12\n\nupdated_at\x18\r \x01(\x03\x12\x12\n\nupdated_by\x18\x0e \x01(\t\x12\n\n\x02id\x18\x12 \x01(\t\x12\x13\n\x0b\x62rowse_only\x18\x15 \x01(\x08\"\xd0\x01\n\x15\x43reateRegisteredModel\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0c\x63\x61talog_name\x18\x02 \x01(\t\x12\x13\n\x0bschema_name\x18\x03 \x01(\t\x12\x0f\n\x07\x63omment\x18\x05 \x01(\t\x12\x18\n\x10storage_location\x18\x06 \x01(\t\x1aS\n\x08Response\x12G\n\x15registered_model_info\x18\x01 \x01(\x0b\x32(.mlflow.unitycatalog.RegisteredModelInfo\"E\n\x15\x44\x65leteRegisteredModel\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\r\n\x05\x66orce\x18\x02 \x01(\x08\x1a\n\n\x08Response\"\x82\x01\n\x12GetRegisteredModel\x12\x17\n\tfull_name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1aS\n\x08Response\x12G\n\x15registered_model_info\x18\x01 \x01(\x0b\x32(.mlflow.unitycatalog.RegisteredModelInfo\"\xa2\x01\n\x15UpdateRegisteredModel\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x10\n\x08new_name\x18\x03 \x01(\t\x12\x0f\n\x07\x63omment\x18\x02 \x01(\t\x1aS\n\x08Response\x12G\n\x15registered_model_info\x18\x01 \x01(\x0b\x32(.mlflow.unitycatalog.RegisteredModelInfo\"\xd4\x01\n\x14ListRegisteredModels\x12\x14\n\x0c\x63\x61talog_name\x18\x01 \x01(\t\x12\x13\n\x0bschema_name\x18\x02 \x01(\t\x12\x13\n\x0bmax_results\x18\x05 \x01(\x03\x12\x12\n\npage_token\x18\x06 \x01(\t\x1ah\n\x08Response\x12\x43\n\x11registered_models\x18\x01 \x03(\x0b\x32(.mlflow.unitycatalog.RegisteredModelInfo\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"\xc2\x02\n\x10ModelVersionInfo\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x14\n\x0c\x63\x61talog_name\x18\x02 \x01(\t\x12\x13\n\x0bschema_name\x18\x03 \x01(\t\x12\x0e\n\x06source\x18\x05 \x01(\t\x12\x0f\n\x07\x63omment\x18\x04 \x01(\t\x12\x0e\n\x06run_id\x18\x06 \x01(\t\x12\x37\n\x06status\x18\x08 \x01(\x0e\x32\'.mlflow.unitycatalog.ModelVersionStatus\x12\x0f\n\x07version\x18\t \x01(\x03\x12\x18\n\x10storage_location\x18\n \x01(\t\x12\x12\n\ncreated_at\x18\x0c \x01(\x03\x12\x12\n\ncreated_by\x18\r \x01(\t\x12\x12\n\nupdated_at\x18\x0e \x01(\x03\x12\x12\n\nupdated_by\x18\x0f \x01(\t\x12\n\n\x02id\x18\x10 \x01(\t\"\xd3\x01\n\x12\x43reateModelVersion\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x14\n\x0c\x63\x61talog_name\x18\x02 \x01(\t\x12\x13\n\x0bschema_name\x18\x03 \x01(\t\x12\x0e\n\x06source\x18\x04 \x01(\t\x12\x0e\n\x06run_id\x18\x05 \x01(\t\x12\x0f\n\x07\x63omment\x18\x06 \x01(\t\x1aM\n\x08Response\x12\x41\n\x12model_version_info\x18\x01 \x01(\x0b\x32%.mlflow.unitycatalog.ModelVersionInfo\"D\n\x12\x44\x65leteModelVersion\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x03\x1a\n\n\x08Response\"\x89\x01\n\x14\x46inalizeModelVersion\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x03\x1aM\n\x08Response\x12\x41\n\x12model_version_info\x18\x01 \x01(\x0b\x32%.mlflow.unitycatalog.ModelVersionInfo\"\x84\x01\n\x0fGetModelVersion\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x03\x1aM\n\x08Response\x12\x41\n\x12model_version_info\x18\x01 \x01(\x0b\x32%.mlflow.unitycatalog.ModelVersionInfo\"\x98\x01\n\x12UpdateModelVersion\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x03\x12\x0f\n\x07\x63omment\x18\x03 \x01(\t\x1aM\n\x08Response\x12\x41\n\x12model_version_info\x18\x01 \x01(\x0b\x32%.mlflow.unitycatalog.ModelVersionInfo\"\xb3\x01\n\x11ListModelVersions\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x13\n\x0bmax_results\x18\x04 \x01(\x03\x12\x12\n\npage_token\x18\x05 \x01(\t\x1a\x62\n\x08Response\x12=\n\x0emodel_versions\x18\x01 \x03(\x0b\x32%.mlflow.unitycatalog.ModelVersionInfo\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"\xff\x01\n\x14TemporaryCredentials\x12\x41\n\x14\x61ws_temp_credentials\x18\x02 \x01(\x0b\x32#.mlflow.unitycatalog.AwsCredentials\x12N\n\x19\x61zure_user_delegation_sas\x18\x03 \x01(\x0b\x32+.mlflow.unitycatalog.AzureUserDelegationSAS\x12;\n\x0fgcp_oauth_token\x18\x04 \x01(\x0b\x32\".mlflow.unitycatalog.GcpOauthToken\x12\x17\n\x0f\x65xpiration_time\x18\x01 \x01(\x03\"Y\n\x0e\x41wsCredentials\x12\x15\n\raccess_key_id\x18\x01 \x01(\t\x12\x19\n\x11secret_access_key\x18\x02 \x01(\t\x12\x15\n\rsession_token\x18\x03 \x01(\t\"+\n\x16\x41zureUserDelegationSAS\x12\x11\n\tsas_token\x18\x01 \x01(\t\"$\n\rGcpOauthToken\x12\x13\n\x0boauth_token\x18\x01 \x01(\t\"\x84\x02\n\'GenerateTemporaryModelVersionCredential\x12\x14\n\x0c\x63\x61talog_name\x18\x01 \x01(\t\x12\x13\n\x0bschema_name\x18\x02 \x01(\t\x12\x12\n\nmodel_name\x18\x03 \x01(\t\x12\x0f\n\x07version\x18\x04 \x01(\x03\x12=\n\toperation\x18\x05 \x01(\x0e\x32*.mlflow.unitycatalog.ModelVersionOperation\x1aJ\n\x08Response\x12>\n\x0b\x63redentials\x18\x01 \x01(\x0b\x32).mlflow.unitycatalog.TemporaryCredentials*t\n\x12ModelVersionStatus\x12 \n\x1cMODEL_VERSION_STATUS_UNKNOWN\x10\x00\x12\x18\n\x14PENDING_REGISTRATION\x10\x01\x12\x17\n\x13\x46\x41ILED_REGISTRATION\x10\x02\x12\t\n\x05READY\x10\x03*r\n\x15ModelVersionOperation\x12#\n\x1fUNKNOWN_MODEL_VERSION_OPERATION\x10\x00\x12\x16\n\x12READ_MODEL_VERSION\x10\x01\x12\x1c\n\x18READ_WRITE_MODEL_VERSION\x10\x02\x42\x31\n\'com.databricks.api.proto.managedcatalog\xa0\x01\x01\xe2?\x02\x10\x01')

  _MODELVERSIONSTATUS = DESCRIPTOR.enum_types_by_name['ModelVersionStatus']
  ModelVersionStatus = enum_type_wrapper.EnumTypeWrapper(_MODELVERSIONSTATUS)
  _MODELVERSIONOPERATION = DESCRIPTOR.enum_types_by_name['ModelVersionOperation']
  ModelVersionOperation = enum_type_wrapper.EnumTypeWrapper(_MODELVERSIONOPERATION)
  MODEL_VERSION_STATUS_UNKNOWN = 0
  PENDING_REGISTRATION = 1
  FAILED_REGISTRATION = 2
  READY = 3
  UNKNOWN_MODEL_VERSION_OPERATION = 0
  READ_MODEL_VERSION = 1
  READ_WRITE_MODEL_VERSION = 2


  _REGISTEREDMODELINFO = DESCRIPTOR.message_types_by_name['RegisteredModelInfo']
  _CREATEREGISTEREDMODEL = DESCRIPTOR.message_types_by_name['CreateRegisteredModel']
  _CREATEREGISTEREDMODEL_RESPONSE = _CREATEREGISTEREDMODEL.nested_types_by_name['Response']
  _DELETEREGISTEREDMODEL = DESCRIPTOR.message_types_by_name['DeleteRegisteredModel']
  _DELETEREGISTEREDMODEL_RESPONSE = _DELETEREGISTEREDMODEL.nested_types_by_name['Response']
  _GETREGISTEREDMODEL = DESCRIPTOR.message_types_by_name['GetRegisteredModel']
  _GETREGISTEREDMODEL_RESPONSE = _GETREGISTEREDMODEL.nested_types_by_name['Response']
  _UPDATEREGISTEREDMODEL = DESCRIPTOR.message_types_by_name['UpdateRegisteredModel']
  _UPDATEREGISTEREDMODEL_RESPONSE = _UPDATEREGISTEREDMODEL.nested_types_by_name['Response']
  _LISTREGISTEREDMODELS = DESCRIPTOR.message_types_by_name['ListRegisteredModels']
  _LISTREGISTEREDMODELS_RESPONSE = _LISTREGISTEREDMODELS.nested_types_by_name['Response']
  _MODELVERSIONINFO = DESCRIPTOR.message_types_by_name['ModelVersionInfo']
  _CREATEMODELVERSION = DESCRIPTOR.message_types_by_name['CreateModelVersion']
  _CREATEMODELVERSION_RESPONSE = _CREATEMODELVERSION.nested_types_by_name['Response']
  _DELETEMODELVERSION = DESCRIPTOR.message_types_by_name['DeleteModelVersion']
  _DELETEMODELVERSION_RESPONSE = _DELETEMODELVERSION.nested_types_by_name['Response']
  _FINALIZEMODELVERSION = DESCRIPTOR.message_types_by_name['FinalizeModelVersion']
  _FINALIZEMODELVERSION_RESPONSE = _FINALIZEMODELVERSION.nested_types_by_name['Response']
  _GETMODELVERSION = DESCRIPTOR.message_types_by_name['GetModelVersion']
  _GETMODELVERSION_RESPONSE = _GETMODELVERSION.nested_types_by_name['Response']
  _UPDATEMODELVERSION = DESCRIPTOR.message_types_by_name['UpdateModelVersion']
  _UPDATEMODELVERSION_RESPONSE = _UPDATEMODELVERSION.nested_types_by_name['Response']
  _LISTMODELVERSIONS = DESCRIPTOR.message_types_by_name['ListModelVersions']
  _LISTMODELVERSIONS_RESPONSE = _LISTMODELVERSIONS.nested_types_by_name['Response']
  _TEMPORARYCREDENTIALS = DESCRIPTOR.message_types_by_name['TemporaryCredentials']
  _AWSCREDENTIALS = DESCRIPTOR.message_types_by_name['AwsCredentials']
  _AZUREUSERDELEGATIONSAS = DESCRIPTOR.message_types_by_name['AzureUserDelegationSAS']
  _GCPOAUTHTOKEN = DESCRIPTOR.message_types_by_name['GcpOauthToken']
  _GENERATETEMPORARYMODELVERSIONCREDENTIAL = DESCRIPTOR.message_types_by_name['GenerateTemporaryModelVersionCredential']
  _GENERATETEMPORARYMODELVERSIONCREDENTIAL_RESPONSE = _GENERATETEMPORARYMODELVERSIONCREDENTIAL.nested_types_by_name['Response']
  RegisteredModelInfo = _reflection.GeneratedProtocolMessageType('RegisteredModelInfo', (_message.Message,), {
    'DESCRIPTOR' : _REGISTEREDMODELINFO,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.RegisteredModelInfo)
    })
  _sym_db.RegisterMessage(RegisteredModelInfo)

  CreateRegisteredModel = _reflection.GeneratedProtocolMessageType('CreateRegisteredModel', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATEREGISTEREDMODEL_RESPONSE,
      '__module__' : 'unity_catalog_oss_messages_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.CreateRegisteredModel.Response)
      })
    ,
    'DESCRIPTOR' : _CREATEREGISTEREDMODEL,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.CreateRegisteredModel)
    })
  _sym_db.RegisterMessage(CreateRegisteredModel)
  _sym_db.RegisterMessage(CreateRegisteredModel.Response)

  DeleteRegisteredModel = _reflection.GeneratedProtocolMessageType('DeleteRegisteredModel', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEREGISTEREDMODEL_RESPONSE,
      '__module__' : 'unity_catalog_oss_messages_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.DeleteRegisteredModel.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEREGISTEREDMODEL,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.DeleteRegisteredModel)
    })
  _sym_db.RegisterMessage(DeleteRegisteredModel)
  _sym_db.RegisterMessage(DeleteRegisteredModel.Response)

  GetRegisteredModel = _reflection.GeneratedProtocolMessageType('GetRegisteredModel', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETREGISTEREDMODEL_RESPONSE,
      '__module__' : 'unity_catalog_oss_messages_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.GetRegisteredModel.Response)
      })
    ,
    'DESCRIPTOR' : _GETREGISTEREDMODEL,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.GetRegisteredModel)
    })
  _sym_db.RegisterMessage(GetRegisteredModel)
  _sym_db.RegisterMessage(GetRegisteredModel.Response)

  UpdateRegisteredModel = _reflection.GeneratedProtocolMessageType('UpdateRegisteredModel', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _UPDATEREGISTEREDMODEL_RESPONSE,
      '__module__' : 'unity_catalog_oss_messages_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.UpdateRegisteredModel.Response)
      })
    ,
    'DESCRIPTOR' : _UPDATEREGISTEREDMODEL,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.UpdateRegisteredModel)
    })
  _sym_db.RegisterMessage(UpdateRegisteredModel)
  _sym_db.RegisterMessage(UpdateRegisteredModel.Response)

  ListRegisteredModels = _reflection.GeneratedProtocolMessageType('ListRegisteredModels', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LISTREGISTEREDMODELS_RESPONSE,
      '__module__' : 'unity_catalog_oss_messages_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.ListRegisteredModels.Response)
      })
    ,
    'DESCRIPTOR' : _LISTREGISTEREDMODELS,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.ListRegisteredModels)
    })
  _sym_db.RegisterMessage(ListRegisteredModels)
  _sym_db.RegisterMessage(ListRegisteredModels.Response)

  ModelVersionInfo = _reflection.GeneratedProtocolMessageType('ModelVersionInfo', (_message.Message,), {
    'DESCRIPTOR' : _MODELVERSIONINFO,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.ModelVersionInfo)
    })
  _sym_db.RegisterMessage(ModelVersionInfo)

  CreateModelVersion = _reflection.GeneratedProtocolMessageType('CreateModelVersion', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATEMODELVERSION_RESPONSE,
      '__module__' : 'unity_catalog_oss_messages_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.CreateModelVersion.Response)
      })
    ,
    'DESCRIPTOR' : _CREATEMODELVERSION,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.CreateModelVersion)
    })
  _sym_db.RegisterMessage(CreateModelVersion)
  _sym_db.RegisterMessage(CreateModelVersion.Response)

  DeleteModelVersion = _reflection.GeneratedProtocolMessageType('DeleteModelVersion', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEMODELVERSION_RESPONSE,
      '__module__' : 'unity_catalog_oss_messages_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.DeleteModelVersion.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEMODELVERSION,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.DeleteModelVersion)
    })
  _sym_db.RegisterMessage(DeleteModelVersion)
  _sym_db.RegisterMessage(DeleteModelVersion.Response)

  FinalizeModelVersion = _reflection.GeneratedProtocolMessageType('FinalizeModelVersion', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _FINALIZEMODELVERSION_RESPONSE,
      '__module__' : 'unity_catalog_oss_messages_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.FinalizeModelVersion.Response)
      })
    ,
    'DESCRIPTOR' : _FINALIZEMODELVERSION,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.FinalizeModelVersion)
    })
  _sym_db.RegisterMessage(FinalizeModelVersion)
  _sym_db.RegisterMessage(FinalizeModelVersion.Response)

  GetModelVersion = _reflection.GeneratedProtocolMessageType('GetModelVersion', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETMODELVERSION_RESPONSE,
      '__module__' : 'unity_catalog_oss_messages_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.GetModelVersion.Response)
      })
    ,
    'DESCRIPTOR' : _GETMODELVERSION,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.GetModelVersion)
    })
  _sym_db.RegisterMessage(GetModelVersion)
  _sym_db.RegisterMessage(GetModelVersion.Response)

  UpdateModelVersion = _reflection.GeneratedProtocolMessageType('UpdateModelVersion', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _UPDATEMODELVERSION_RESPONSE,
      '__module__' : 'unity_catalog_oss_messages_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.UpdateModelVersion.Response)
      })
    ,
    'DESCRIPTOR' : _UPDATEMODELVERSION,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.UpdateModelVersion)
    })
  _sym_db.RegisterMessage(UpdateModelVersion)
  _sym_db.RegisterMessage(UpdateModelVersion.Response)

  ListModelVersions = _reflection.GeneratedProtocolMessageType('ListModelVersions', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LISTMODELVERSIONS_RESPONSE,
      '__module__' : 'unity_catalog_oss_messages_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.ListModelVersions.Response)
      })
    ,
    'DESCRIPTOR' : _LISTMODELVERSIONS,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.ListModelVersions)
    })
  _sym_db.RegisterMessage(ListModelVersions)
  _sym_db.RegisterMessage(ListModelVersions.Response)

  TemporaryCredentials = _reflection.GeneratedProtocolMessageType('TemporaryCredentials', (_message.Message,), {
    'DESCRIPTOR' : _TEMPORARYCREDENTIALS,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.TemporaryCredentials)
    })
  _sym_db.RegisterMessage(TemporaryCredentials)

  AwsCredentials = _reflection.GeneratedProtocolMessageType('AwsCredentials', (_message.Message,), {
    'DESCRIPTOR' : _AWSCREDENTIALS,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.AwsCredentials)
    })
  _sym_db.RegisterMessage(AwsCredentials)

  AzureUserDelegationSAS = _reflection.GeneratedProtocolMessageType('AzureUserDelegationSAS', (_message.Message,), {
    'DESCRIPTOR' : _AZUREUSERDELEGATIONSAS,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.AzureUserDelegationSAS)
    })
  _sym_db.RegisterMessage(AzureUserDelegationSAS)

  GcpOauthToken = _reflection.GeneratedProtocolMessageType('GcpOauthToken', (_message.Message,), {
    'DESCRIPTOR' : _GCPOAUTHTOKEN,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.GcpOauthToken)
    })
  _sym_db.RegisterMessage(GcpOauthToken)

  GenerateTemporaryModelVersionCredential = _reflection.GeneratedProtocolMessageType('GenerateTemporaryModelVersionCredential', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GENERATETEMPORARYMODELVERSIONCREDENTIAL_RESPONSE,
      '__module__' : 'unity_catalog_oss_messages_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.GenerateTemporaryModelVersionCredential.Response)
      })
    ,
    'DESCRIPTOR' : _GENERATETEMPORARYMODELVERSIONCREDENTIAL,
    '__module__' : 'unity_catalog_oss_messages_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.unitycatalog.GenerateTemporaryModelVersionCredential)
    })
  _sym_db.RegisterMessage(GenerateTemporaryModelVersionCredential)
  _sym_db.RegisterMessage(GenerateTemporaryModelVersionCredential.Response)

  if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b'\n\'com.databricks.api.proto.managedcatalog\240\001\001\342?\002\020\001'
    _GETREGISTEREDMODEL.fields_by_name['full_name']._options = None
    _GETREGISTEREDMODEL.fields_by_name['full_name']._serialized_options = b'\370\206\031\001'
    _MODELVERSIONSTATUS._serialized_start=3065
    _MODELVERSIONSTATUS._serialized_end=3181
    _MODELVERSIONOPERATION._serialized_start=3183
    _MODELVERSIONOPERATION._serialized_end=3297
    _REGISTEREDMODELINFO._serialized_start=99
    _REGISTEREDMODELINFO._serialized_end=352
    _CREATEREGISTEREDMODEL._serialized_start=355
    _CREATEREGISTEREDMODEL._serialized_end=563
    _CREATEREGISTEREDMODEL_RESPONSE._serialized_start=480
    _CREATEREGISTEREDMODEL_RESPONSE._serialized_end=563
    _DELETEREGISTEREDMODEL._serialized_start=565
    _DELETEREGISTEREDMODEL._serialized_end=634
    _DELETEREGISTEREDMODEL_RESPONSE._serialized_start=480
    _DELETEREGISTEREDMODEL_RESPONSE._serialized_end=490
    _GETREGISTEREDMODEL._serialized_start=637
    _GETREGISTEREDMODEL._serialized_end=767
    _GETREGISTEREDMODEL_RESPONSE._serialized_start=480
    _GETREGISTEREDMODEL_RESPONSE._serialized_end=563
    _UPDATEREGISTEREDMODEL._serialized_start=770
    _UPDATEREGISTEREDMODEL._serialized_end=932
    _UPDATEREGISTEREDMODEL_RESPONSE._serialized_start=480
    _UPDATEREGISTEREDMODEL_RESPONSE._serialized_end=563
    _LISTREGISTEREDMODELS._serialized_start=935
    _LISTREGISTEREDMODELS._serialized_end=1147
    _LISTREGISTEREDMODELS_RESPONSE._serialized_start=1043
    _LISTREGISTEREDMODELS_RESPONSE._serialized_end=1147
    _MODELVERSIONINFO._serialized_start=1150
    _MODELVERSIONINFO._serialized_end=1472
    _CREATEMODELVERSION._serialized_start=1475
    _CREATEMODELVERSION._serialized_end=1686
    _CREATEMODELVERSION_RESPONSE._serialized_start=1609
    _CREATEMODELVERSION_RESPONSE._serialized_end=1686
    _DELETEMODELVERSION._serialized_start=1688
    _DELETEMODELVERSION._serialized_end=1756
    _DELETEMODELVERSION_RESPONSE._serialized_start=480
    _DELETEMODELVERSION_RESPONSE._serialized_end=490
    _FINALIZEMODELVERSION._serialized_start=1759
    _FINALIZEMODELVERSION._serialized_end=1896
    _FINALIZEMODELVERSION_RESPONSE._serialized_start=1609
    _FINALIZEMODELVERSION_RESPONSE._serialized_end=1686
    _GETMODELVERSION._serialized_start=1899
    _GETMODELVERSION._serialized_end=2031
    _GETMODELVERSION_RESPONSE._serialized_start=1609
    _GETMODELVERSION_RESPONSE._serialized_end=1686
    _UPDATEMODELVERSION._serialized_start=2034
    _UPDATEMODELVERSION._serialized_end=2186
    _UPDATEMODELVERSION_RESPONSE._serialized_start=1609
    _UPDATEMODELVERSION_RESPONSE._serialized_end=1686
    _LISTMODELVERSIONS._serialized_start=2189
    _LISTMODELVERSIONS._serialized_end=2368
    _LISTMODELVERSIONS_RESPONSE._serialized_start=2270
    _LISTMODELVERSIONS_RESPONSE._serialized_end=2368
    _TEMPORARYCREDENTIALS._serialized_start=2371
    _TEMPORARYCREDENTIALS._serialized_end=2626
    _AWSCREDENTIALS._serialized_start=2628
    _AWSCREDENTIALS._serialized_end=2717
    _AZUREUSERDELEGATIONSAS._serialized_start=2719
    _AZUREUSERDELEGATIONSAS._serialized_end=2762
    _GCPOAUTHTOKEN._serialized_start=2764
    _GCPOAUTHTOKEN._serialized_end=2800
    _GENERATETEMPORARYMODELVERSIONCREDENTIAL._serialized_start=2803
    _GENERATETEMPORARYMODELVERSIONCREDENTIAL._serialized_end=3063
    _GENERATETEMPORARYMODELVERSIONCREDENTIAL_RESPONSE._serialized_start=2989
    _GENERATETEMPORARYMODELVERSIONCREDENTIAL_RESPONSE._serialized_end=3063
  # @@protoc_insertion_point(module_scope)




import logging
import os

_logger = logging.getLogger(__name__)


def get_git_repo_url(path: str) -> str | None:
    """
    Obtains the url of the git repository associated with the specified path,
    returning ``None`` if the path does not correspond to a git repository.
    """
    try:
        from git import Repo
    except ImportError as e:
        _logger.warning(
            "Failed to import Git (the Git executable is probably not on your PATH),"
            " so Git SHA is not available. Error: %s",
            e,
        )
        return None

    try:
        repo = Repo(path, search_parent_directories=True)
        return next((remote.url for remote in repo.remotes), None)
    except Exception:
        return None


def get_git_commit(path: str) -> str | None:
    """
    Obtains the hash of the latest commit on the current branch of the git repository associated
    with the specified path, returning ``None`` if the path does not correspond to a git
    repository.
    """
    try:
        from git import Repo
    except ImportError as e:
        _logger.warning(
            "Failed to import Git (the Git executable is probably not on your PATH),"
            " so Git SHA is not available. Error: %s",
            e,
        )
        return None
    try:
        if os.path.isfile(path):
            path = os.path.dirname(os.path.abspath(path))
        repo = Repo(path, search_parent_directories=True)
        if path in repo.ignored(path):
            return None
        return repo.head.commit.hexsha
    except Exception:
        return None


def get_git_branch(path: str) -> str | None:
    """
    Obtains the name of the current branch of the git repository associated with the specified
    path, returning ``None`` if the path does not correspond to a git repository.
    """
    try:
        from git import Repo
    except ImportError as e:
        _logger.warning(
            "Failed to import Git (the Git executable is probably not on your PATH),"
            " so Git SHA is not available. Error: %s",
            e,
        )
        return None

    try:
        if os.path.isfile(path):
            path = os.path.dirname(path)
        repo = Repo(path, search_parent_directories=True)
        return repo.active_branch.name
    except Exception:
        return None




import google.protobuf
from packaging.version import Version
if Version(google.protobuf.__version__).major >= 5:
  # -*- coding: utf-8 -*-
  # Generated by the protocol buffer compiler.  DO NOT EDIT!
  # source: databricks_uc_registry_service.proto
  # Protobuf Python Version: 5.26.0
  """Generated protocol buffer code."""
  from google.protobuf import descriptor as _descriptor
  from google.protobuf import descriptor_pool as _descriptor_pool
  from google.protobuf import symbol_database as _symbol_database
  from google.protobuf.internal import builder as _builder
  # @@protoc_insertion_point(imports)

  _sym_db = _symbol_database.Default()


  from . import databricks_pb2 as databricks__pb2
  from . import databricks_uc_registry_messages_pb2 as databricks_uc_registry_messages_pb2
  from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2


  DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$databricks_uc_registry_service.proto\x12\x16mlflow.ucmodelregistry\x1a\x10\x64\x61tabricks.proto\x1a%databricks_uc_registry_messages.proto\x1a\x15scalapb/scalapb.proto2\xc0,\n\x16UcModelRegistryService\x12\xf2\x01\n\x15\x43reateRegisteredModel\x12\x34.mlflow.ucmodelregistry.CreateRegisteredModelRequest\x1a\x35.mlflow.ucmodelregistry.CreateRegisteredModelResponse\"l\xf2\x86\x19h\n<\n\x04POST\x12./mlflow/unity-catalog/registered-models/create\x1a\x04\x08\x02\x10\x00\x10\x03*&(Unity Catalog) Create RegisteredModel\x12\xf3\x01\n\x15UpdateRegisteredModel\x12\x34.mlflow.ucmodelregistry.UpdateRegisteredModelRequest\x1a\x35.mlflow.ucmodelregistry.UpdateRegisteredModelResponse\"m\xf2\x86\x19i\n=\n\x05PATCH\x12./mlflow/unity-catalog/registered-models/update\x1a\x04\x08\x02\x10\x00\x10\x03*&(Unity Catalog) Update RegisteredModel\x12\xf4\x01\n\x15\x44\x65leteRegisteredModel\x12\x34.mlflow.ucmodelregistry.DeleteRegisteredModelRequest\x1a\x35.mlflow.ucmodelregistry.DeleteRegisteredModelResponse\"n\xf2\x86\x19j\n>\n\x06\x44\x45LETE\x12./mlflow/unity-catalog/registered-models/delete\x1a\x04\x08\x02\x10\x00\x10\x03*&(Unity Catalog) Delete RegisteredModel\x12\xe2\x01\n\x12GetRegisteredModel\x12\x31.mlflow.ucmodelregistry.GetRegisteredModelRequest\x1a\x32.mlflow.ucmodelregistry.GetRegisteredModelResponse\"e\xf2\x86\x19\x61\n8\n\x03GET\x12+/mlflow/unity-catalog/registered-models/get\x1a\x04\x08\x02\x10\x00\x10\x03*#(Unity Catalog) Get RegisteredModel\x12\xf5\x01\n\x16SearchRegisteredModels\x12\x35.mlflow.ucmodelregistry.SearchRegisteredModelsRequest\x1a\x36.mlflow.ucmodelregistry.SearchRegisteredModelsResponse\"l\xf2\x86\x19h\n;\n\x03GET\x12./mlflow/unity-catalog/registered-models/search\x1a\x04\x08\x02\x10\x00\x10\x03*\'(Unity Catalog) Search RegisteredModels\x12\xe3\x01\n\x12\x43reateModelVersion\x12\x31.mlflow.ucmodelregistry.CreateModelVersionRequest\x1a\x32.mlflow.ucmodelregistry.CreateModelVersionResponse\"f\xf2\x86\x19\x62\n9\n\x04POST\x12+/mlflow/unity-catalog/model-versions/create\x1a\x04\x08\x02\x10\x00\x10\x03*#(Unity Catalog) Create ModelVersion\x12\xe4\x01\n\x12UpdateModelVersion\x12\x31.mlflow.ucmodelregistry.UpdateModelVersionRequest\x1a\x32.mlflow.ucmodelregistry.UpdateModelVersionResponse\"g\xf2\x86\x19\x63\n:\n\x05PATCH\x12+/mlflow/unity-catalog/model-versions/update\x1a\x04\x08\x02\x10\x00\x10\x03*#(Unity Catalog) Update ModelVersion\x12\xe5\x01\n\x12\x44\x65leteModelVersion\x12\x31.mlflow.ucmodelregistry.DeleteModelVersionRequest\x1a\x32.mlflow.ucmodelregistry.DeleteModelVersionResponse\"h\xf2\x86\x19\x64\n;\n\x06\x44\x45LETE\x12+/mlflow/unity-catalog/model-versions/delete\x1a\x04\x08\x02\x10\x00\x10\x03*#(Unity Catalog) Delete ModelVersion\x12\xd3\x01\n\x0fGetModelVersion\x12..mlflow.ucmodelregistry.GetModelVersionRequest\x1a/.mlflow.ucmodelregistry.GetModelVersionResponse\"_\xf2\x86\x19[\n5\n\x03GET\x12(/mlflow/unity-catalog/model-versions/get\x1a\x04\x08\x02\x10\x00\x10\x03* (Unity Catalog) Get ModelVersion\x12\xe6\x01\n\x13SearchModelVersions\x12\x32.mlflow.ucmodelregistry.SearchModelVersionsRequest\x1a\x33.mlflow.ucmodelregistry.SearchModelVersionsResponse\"f\xf2\x86\x19\x62\n8\n\x03GET\x12+/mlflow/unity-catalog/model-versions/search\x1a\x04\x08\x02\x10\x00\x10\x03*$(Unity Catalog) Search ModelVersions\x12\xd8\x02\n(GenerateTemporaryModelVersionCredentials\x12G.mlflow.ucmodelregistry.GenerateTemporaryModelVersionCredentialsRequest\x1aH.mlflow.ucmodelregistry.GenerateTemporaryModelVersionCredentialsResponse\"\x98\x01\xf2\x86\x19\x93\x01\nQ\n\x04POST\x12\x43/mlflow/unity-catalog/model-versions/generate-temporary-credentials\x1a\x04\x08\x02\x10\x00\x10\x03*<(Unity Catalog) Generate Temporary Model Version Credentials\x12\x9e\x02\n\x1aGetModelVersionDownloadUri\x12\x39.mlflow.ucmodelregistry.GetModelVersionDownloadUriRequest\x1a:.mlflow.ucmodelregistry.GetModelVersionDownloadUriResponse\"\x88\x01\xf2\x86\x19\x83\x01\nB\n\x03GET\x12\x35/mlflow/unity-catalog/model-versions/get-download-uri\x1a\x04\x08\x02\x10\x00\x10\x03*;(Unity Catalog) Get Download URI For ModelVersion Artifacts\x12\xee\x01\n\x14\x46inalizeModelVersion\x12\x33.mlflow.ucmodelregistry.FinalizeModelVersionRequest\x1a\x34.mlflow.ucmodelregistry.FinalizeModelVersionResponse\"k\xf2\x86\x19g\n;\n\x04POST\x12-/mlflow/unity-catalog/model-versions/finalize\x1a\x04\x08\x02\x10\x00\x10\x03*&(Unity Catalog) Finalize Model Version\x12\xfa\x01\n\x17SetRegisteredModelAlias\x12\x36.mlflow.ucmodelregistry.SetRegisteredModelAliasRequest\x1a\x37.mlflow.ucmodelregistry.SetRegisteredModelAliasResponse\"n\xf2\x86\x19j\n;\n\x04POST\x12-/mlflow/unity-catalog/registered-models/alias\x1a\x04\x08\x02\x10\x00\x10\x03*)(Unity Catalog) Set RegisteredModel Alias\x12\x88\x02\n\x1a\x44\x65leteRegisteredModelAlias\x12\x39.mlflow.ucmodelregistry.DeleteRegisteredModelAliasRequest\x1a:.mlflow.ucmodelregistry.DeleteRegisteredModelAliasResponse\"s\xf2\x86\x19o\n=\n\x06\x44\x45LETE\x12-/mlflow/unity-catalog/registered-models/alias\x1a\x04\x08\x02\x10\x00\x10\x03*,(Unity Catalog) Delete RegisteredModel Alias\x12\xf6\x01\n\x16GetModelVersionByAlias\x12\x35.mlflow.ucmodelregistry.GetModelVersionByAliasRequest\x1a\x36.mlflow.ucmodelregistry.GetModelVersionByAliasResponse\"m\xf2\x86\x19i\n:\n\x03GET\x12-/mlflow/unity-catalog/registered-models/alias\x1a\x04\x08\x02\x10\x00\x10\x03*)(Unity Catalog) Get ModelVersion By Alias\x12\xf4\x01\n\x15SetRegisteredModelTag\x12\x34.mlflow.ucmodelregistry.SetRegisteredModelTagRequest\x1a\x35.mlflow.ucmodelregistry.SetRegisteredModelTagResponse\"n\xf2\x86\x19j\n=\n\x04POST\x12//mlflow/unity-catalog/registered-models/set-tag\x1a\x04\x08\x02\x10\x00\x10\x03*\'(Unity Catalog) Set RegisteredModel Tag\x12\x85\x02\n\x18\x44\x65leteRegisteredModelTag\x12\x37.mlflow.ucmodelregistry.DeleteRegisteredModelTagRequest\x1a\x38.mlflow.ucmodelregistry.DeleteRegisteredModelTagResponse\"v\xf2\x86\x19r\nB\n\x06\x44\x45LETE\x12\x32/mlflow/unity-catalog/registered-models/delete-tag\x1a\x04\x08\x02\x10\x00\x10\x03**(Unity Catalog) Delete RegisteredModel Tag\x12\xe5\x01\n\x12SetModelVersionTag\x12\x31.mlflow.ucmodelregistry.SetModelVersionTagRequest\x1a\x32.mlflow.ucmodelregistry.SetModelVersionTagResponse\"h\xf2\x86\x19\x64\n:\n\x04POST\x12,/mlflow/unity-catalog/model-versions/set-tag\x1a\x04\x08\x02\x10\x00\x10\x03*$(Unity Catalog) Set ModelVersion Tag\x12\xf6\x01\n\x15\x44\x65leteModelVersionTag\x12\x34.mlflow.ucmodelregistry.DeleteModelVersionTagRequest\x1a\x35.mlflow.ucmodelregistry.DeleteModelVersionTagResponse\"p\xf2\x86\x19l\n?\n\x06\x44\x45LETE\x12//mlflow/unity-catalog/model-versions/delete-tag\x1a\x04\x08\x02\x10\x00\x10\x03*\'(Unity Catalog) Delete ModelVersion Tag\x12\xfe\x01\n\x17\x45mitModelVersionLineage\x12\x36.mlflow.ucmodelregistry.EmitModelVersionLineageRequest\x1a\x37.mlflow.ucmodelregistry.EmitModelVersionLineageResponse\"r\xf2\x86\x19n\n?\n\x04POST\x12\x31/mlflow/unity-catalog/model-versions/emit-lineage\x1a\x04\x08\x02\x10\x00\x10\x03*)(Unity Catalog) Emit ModelVersion Lineage\x12\xa3\x03\n.IsDatabricksSdkModelsArtifactRepositoryEnabled\x12M.mlflow.ucmodelregistry.IsDatabricksSdkModelsArtifactRepositoryEnabledRequest\x1aN.mlflow.ucmodelregistry.IsDatabricksSdkModelsArtifactRepositoryEnabledResponse\"\xd1\x01\xf2\x86\x19\xcc\x01\ni\n\x03GET\x12\\/mlflow/unity-catalog/registered-models:is-databricks-sdk-models-artifact-repository-enabled\x1a\x04\x08\x02\x10\x00\x10\x03*](Unity Catalog) Use DatabricksSdkModelsArtifactRepository in mlflow client for model registryB5\n(com.databricks.api.proto.ucmodelregistry\x90\x01\x01\xa0\x01\x01\xe2?\x02\x10\x01')

  _globals = globals()
  _builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
  _builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'databricks_uc_registry_service_pb2', _globals)
  if not _descriptor._USE_C_DESCRIPTORS:
    _globals['DESCRIPTOR']._loaded_options = None
    _globals['DESCRIPTOR']._serialized_options = b'\n(com.databricks.api.proto.ucmodelregistry\220\001\001\240\001\001\342?\002\020\001'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['CreateRegisteredModel']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['CreateRegisteredModel']._serialized_options = b'\362\206\031h\n<\n\004POST\022./mlflow/unity-catalog/registered-models/create\032\004\010\002\020\000\020\003*&(Unity Catalog) Create RegisteredModel'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['UpdateRegisteredModel']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['UpdateRegisteredModel']._serialized_options = b'\362\206\031i\n=\n\005PATCH\022./mlflow/unity-catalog/registered-models/update\032\004\010\002\020\000\020\003*&(Unity Catalog) Update RegisteredModel'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['DeleteRegisteredModel']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['DeleteRegisteredModel']._serialized_options = b'\362\206\031j\n>\n\006DELETE\022./mlflow/unity-catalog/registered-models/delete\032\004\010\002\020\000\020\003*&(Unity Catalog) Delete RegisteredModel'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['GetRegisteredModel']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['GetRegisteredModel']._serialized_options = b'\362\206\031a\n8\n\003GET\022+/mlflow/unity-catalog/registered-models/get\032\004\010\002\020\000\020\003*#(Unity Catalog) Get RegisteredModel'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['SearchRegisteredModels']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['SearchRegisteredModels']._serialized_options = b'\362\206\031h\n;\n\003GET\022./mlflow/unity-catalog/registered-models/search\032\004\010\002\020\000\020\003*\'(Unity Catalog) Search RegisteredModels'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['CreateModelVersion']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['CreateModelVersion']._serialized_options = b'\362\206\031b\n9\n\004POST\022+/mlflow/unity-catalog/model-versions/create\032\004\010\002\020\000\020\003*#(Unity Catalog) Create ModelVersion'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['UpdateModelVersion']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['UpdateModelVersion']._serialized_options = b'\362\206\031c\n:\n\005PATCH\022+/mlflow/unity-catalog/model-versions/update\032\004\010\002\020\000\020\003*#(Unity Catalog) Update ModelVersion'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['DeleteModelVersion']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['DeleteModelVersion']._serialized_options = b'\362\206\031d\n;\n\006DELETE\022+/mlflow/unity-catalog/model-versions/delete\032\004\010\002\020\000\020\003*#(Unity Catalog) Delete ModelVersion'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['GetModelVersion']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['GetModelVersion']._serialized_options = b'\362\206\031[\n5\n\003GET\022(/mlflow/unity-catalog/model-versions/get\032\004\010\002\020\000\020\003* (Unity Catalog) Get ModelVersion'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['SearchModelVersions']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['SearchModelVersions']._serialized_options = b'\362\206\031b\n8\n\003GET\022+/mlflow/unity-catalog/model-versions/search\032\004\010\002\020\000\020\003*$(Unity Catalog) Search ModelVersions'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['GenerateTemporaryModelVersionCredentials']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['GenerateTemporaryModelVersionCredentials']._serialized_options = b'\362\206\031\223\001\nQ\n\004POST\022C/mlflow/unity-catalog/model-versions/generate-temporary-credentials\032\004\010\002\020\000\020\003*<(Unity Catalog) Generate Temporary Model Version Credentials'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['GetModelVersionDownloadUri']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['GetModelVersionDownloadUri']._serialized_options = b'\362\206\031\203\001\nB\n\003GET\0225/mlflow/unity-catalog/model-versions/get-download-uri\032\004\010\002\020\000\020\003*;(Unity Catalog) Get Download URI For ModelVersion Artifacts'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['FinalizeModelVersion']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['FinalizeModelVersion']._serialized_options = b'\362\206\031g\n;\n\004POST\022-/mlflow/unity-catalog/model-versions/finalize\032\004\010\002\020\000\020\003*&(Unity Catalog) Finalize Model Version'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['SetRegisteredModelAlias']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['SetRegisteredModelAlias']._serialized_options = b'\362\206\031j\n;\n\004POST\022-/mlflow/unity-catalog/registered-models/alias\032\004\010\002\020\000\020\003*)(Unity Catalog) Set RegisteredModel Alias'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['DeleteRegisteredModelAlias']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['DeleteRegisteredModelAlias']._serialized_options = b'\362\206\031o\n=\n\006DELETE\022-/mlflow/unity-catalog/registered-models/alias\032\004\010\002\020\000\020\003*,(Unity Catalog) Delete RegisteredModel Alias'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['GetModelVersionByAlias']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['GetModelVersionByAlias']._serialized_options = b'\362\206\031i\n:\n\003GET\022-/mlflow/unity-catalog/registered-models/alias\032\004\010\002\020\000\020\003*)(Unity Catalog) Get ModelVersion By Alias'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['SetRegisteredModelTag']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['SetRegisteredModelTag']._serialized_options = b'\362\206\031j\n=\n\004POST\022//mlflow/unity-catalog/registered-models/set-tag\032\004\010\002\020\000\020\003*\'(Unity Catalog) Set RegisteredModel Tag'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['DeleteRegisteredModelTag']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['DeleteRegisteredModelTag']._serialized_options = b'\362\206\031r\nB\n\006DELETE\0222/mlflow/unity-catalog/registered-models/delete-tag\032\004\010\002\020\000\020\003**(Unity Catalog) Delete RegisteredModel Tag'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['SetModelVersionTag']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['SetModelVersionTag']._serialized_options = b'\362\206\031d\n:\n\004POST\022,/mlflow/unity-catalog/model-versions/set-tag\032\004\010\002\020\000\020\003*$(Unity Catalog) Set ModelVersion Tag'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['DeleteModelVersionTag']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['DeleteModelVersionTag']._serialized_options = b'\362\206\031l\n?\n\006DELETE\022//mlflow/unity-catalog/model-versions/delete-tag\032\004\010\002\020\000\020\003*\'(Unity Catalog) Delete ModelVersion Tag'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['EmitModelVersionLineage']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['EmitModelVersionLineage']._serialized_options = b'\362\206\031n\n?\n\004POST\0221/mlflow/unity-catalog/model-versions/emit-lineage\032\004\010\002\020\000\020\003*)(Unity Catalog) Emit ModelVersion Lineage'
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['IsDatabricksSdkModelsArtifactRepositoryEnabled']._loaded_options = None
    _globals['_UCMODELREGISTRYSERVICE'].methods_by_name['IsDatabricksSdkModelsArtifactRepositoryEnabled']._serialized_options = b'\362\206\031\314\001\ni\n\003GET\022\\/mlflow/unity-catalog/registered-models:is-databricks-sdk-models-artifact-repository-enabled\032\004\010\002\020\000\020\003*](Unity Catalog) Use DatabricksSdkModelsArtifactRepository in mlflow client for model registry'
    _globals['_UCMODELREGISTRYSERVICE']._serialized_start=145
    _globals['_UCMODELREGISTRYSERVICE']._serialized_end=5841
  _builder.BuildServices(DESCRIPTOR, 'databricks_uc_registry_service_pb2', _globals)
  # @@protoc_insertion_point(module_scope)

else:
  # -*- coding: utf-8 -*-
  # Generated by the protocol buffer compiler.  DO NOT EDIT!
  # source: databricks_uc_registry_service.proto
  """Generated protocol buffer code."""
  from google.protobuf import descriptor as _descriptor
  from google.protobuf import descriptor_pool as _descriptor_pool
  from google.protobuf import message as _message
  from google.protobuf import reflection as _reflection
  from google.protobuf import symbol_database as _symbol_database
  from google.protobuf import service as _service
  from google.protobuf import service_reflection
  # @@protoc_insertion_point(imports)

  _sym_db = _symbol_database.Default()


  from . import databricks_pb2 as databricks__pb2
  from . import databricks_uc_registry_messages_pb2 as databricks_uc_registry_messages_pb2
  from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2


  DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$databricks_uc_registry_service.proto\x12\x16mlflow.ucmodelregistry\x1a\x10\x64\x61tabricks.proto\x1a%databricks_uc_registry_messages.proto\x1a\x15scalapb/scalapb.proto2\xc0,\n\x16UcModelRegistryService\x12\xf2\x01\n\x15\x43reateRegisteredModel\x12\x34.mlflow.ucmodelregistry.CreateRegisteredModelRequest\x1a\x35.mlflow.ucmodelregistry.CreateRegisteredModelResponse\"l\xf2\x86\x19h\n<\n\x04POST\x12./mlflow/unity-catalog/registered-models/create\x1a\x04\x08\x02\x10\x00\x10\x03*&(Unity Catalog) Create RegisteredModel\x12\xf3\x01\n\x15UpdateRegisteredModel\x12\x34.mlflow.ucmodelregistry.UpdateRegisteredModelRequest\x1a\x35.mlflow.ucmodelregistry.UpdateRegisteredModelResponse\"m\xf2\x86\x19i\n=\n\x05PATCH\x12./mlflow/unity-catalog/registered-models/update\x1a\x04\x08\x02\x10\x00\x10\x03*&(Unity Catalog) Update RegisteredModel\x12\xf4\x01\n\x15\x44\x65leteRegisteredModel\x12\x34.mlflow.ucmodelregistry.DeleteRegisteredModelRequest\x1a\x35.mlflow.ucmodelregistry.DeleteRegisteredModelResponse\"n\xf2\x86\x19j\n>\n\x06\x44\x45LETE\x12./mlflow/unity-catalog/registered-models/delete\x1a\x04\x08\x02\x10\x00\x10\x03*&(Unity Catalog) Delete RegisteredModel\x12\xe2\x01\n\x12GetRegisteredModel\x12\x31.mlflow.ucmodelregistry.GetRegisteredModelRequest\x1a\x32.mlflow.ucmodelregistry.GetRegisteredModelResponse\"e\xf2\x86\x19\x61\n8\n\x03GET\x12+/mlflow/unity-catalog/registered-models/get\x1a\x04\x08\x02\x10\x00\x10\x03*#(Unity Catalog) Get RegisteredModel\x12\xf5\x01\n\x16SearchRegisteredModels\x12\x35.mlflow.ucmodelregistry.SearchRegisteredModelsRequest\x1a\x36.mlflow.ucmodelregistry.SearchRegisteredModelsResponse\"l\xf2\x86\x19h\n;\n\x03GET\x12./mlflow/unity-catalog/registered-models/search\x1a\x04\x08\x02\x10\x00\x10\x03*\'(Unity Catalog) Search RegisteredModels\x12\xe3\x01\n\x12\x43reateModelVersion\x12\x31.mlflow.ucmodelregistry.CreateModelVersionRequest\x1a\x32.mlflow.ucmodelregistry.CreateModelVersionResponse\"f\xf2\x86\x19\x62\n9\n\x04POST\x12+/mlflow/unity-catalog/model-versions/create\x1a\x04\x08\x02\x10\x00\x10\x03*#(Unity Catalog) Create ModelVersion\x12\xe4\x01\n\x12UpdateModelVersion\x12\x31.mlflow.ucmodelregistry.UpdateModelVersionRequest\x1a\x32.mlflow.ucmodelregistry.UpdateModelVersionResponse\"g\xf2\x86\x19\x63\n:\n\x05PATCH\x12+/mlflow/unity-catalog/model-versions/update\x1a\x04\x08\x02\x10\x00\x10\x03*#(Unity Catalog) Update ModelVersion\x12\xe5\x01\n\x12\x44\x65leteModelVersion\x12\x31.mlflow.ucmodelregistry.DeleteModelVersionRequest\x1a\x32.mlflow.ucmodelregistry.DeleteModelVersionResponse\"h\xf2\x86\x19\x64\n;\n\x06\x44\x45LETE\x12+/mlflow/unity-catalog/model-versions/delete\x1a\x04\x08\x02\x10\x00\x10\x03*#(Unity Catalog) Delete ModelVersion\x12\xd3\x01\n\x0fGetModelVersion\x12..mlflow.ucmodelregistry.GetModelVersionRequest\x1a/.mlflow.ucmodelregistry.GetModelVersionResponse\"_\xf2\x86\x19[\n5\n\x03GET\x12(/mlflow/unity-catalog/model-versions/get\x1a\x04\x08\x02\x10\x00\x10\x03* (Unity Catalog) Get ModelVersion\x12\xe6\x01\n\x13SearchModelVersions\x12\x32.mlflow.ucmodelregistry.SearchModelVersionsRequest\x1a\x33.mlflow.ucmodelregistry.SearchModelVersionsResponse\"f\xf2\x86\x19\x62\n8\n\x03GET\x12+/mlflow/unity-catalog/model-versions/search\x1a\x04\x08\x02\x10\x00\x10\x03*$(Unity Catalog) Search ModelVersions\x12\xd8\x02\n(GenerateTemporaryModelVersionCredentials\x12G.mlflow.ucmodelregistry.GenerateTemporaryModelVersionCredentialsRequest\x1aH.mlflow.ucmodelregistry.GenerateTemporaryModelVersionCredentialsResponse\"\x98\x01\xf2\x86\x19\x93\x01\nQ\n\x04POST\x12\x43/mlflow/unity-catalog/model-versions/generate-temporary-credentials\x1a\x04\x08\x02\x10\x00\x10\x03*<(Unity Catalog) Generate Temporary Model Version Credentials\x12\x9e\x02\n\x1aGetModelVersionDownloadUri\x12\x39.mlflow.ucmodelregistry.GetModelVersionDownloadUriRequest\x1a:.mlflow.ucmodelregistry.GetModelVersionDownloadUriResponse\"\x88\x01\xf2\x86\x19\x83\x01\nB\n\x03GET\x12\x35/mlflow/unity-catalog/model-versions/get-download-uri\x1a\x04\x08\x02\x10\x00\x10\x03*;(Unity Catalog) Get Download URI For ModelVersion Artifacts\x12\xee\x01\n\x14\x46inalizeModelVersion\x12\x33.mlflow.ucmodelregistry.FinalizeModelVersionRequest\x1a\x34.mlflow.ucmodelregistry.FinalizeModelVersionResponse\"k\xf2\x86\x19g\n;\n\x04POST\x12-/mlflow/unity-catalog/model-versions/finalize\x1a\x04\x08\x02\x10\x00\x10\x03*&(Unity Catalog) Finalize Model Version\x12\xfa\x01\n\x17SetRegisteredModelAlias\x12\x36.mlflow.ucmodelregistry.SetRegisteredModelAliasRequest\x1a\x37.mlflow.ucmodelregistry.SetRegisteredModelAliasResponse\"n\xf2\x86\x19j\n;\n\x04POST\x12-/mlflow/unity-catalog/registered-models/alias\x1a\x04\x08\x02\x10\x00\x10\x03*)(Unity Catalog) Set RegisteredModel Alias\x12\x88\x02\n\x1a\x44\x65leteRegisteredModelAlias\x12\x39.mlflow.ucmodelregistry.DeleteRegisteredModelAliasRequest\x1a:.mlflow.ucmodelregistry.DeleteRegisteredModelAliasResponse\"s\xf2\x86\x19o\n=\n\x06\x44\x45LETE\x12-/mlflow/unity-catalog/registered-models/alias\x1a\x04\x08\x02\x10\x00\x10\x03*,(Unity Catalog) Delete RegisteredModel Alias\x12\xf6\x01\n\x16GetModelVersionByAlias\x12\x35.mlflow.ucmodelregistry.GetModelVersionByAliasRequest\x1a\x36.mlflow.ucmodelregistry.GetModelVersionByAliasResponse\"m\xf2\x86\x19i\n:\n\x03GET\x12-/mlflow/unity-catalog/registered-models/alias\x1a\x04\x08\x02\x10\x00\x10\x03*)(Unity Catalog) Get ModelVersion By Alias\x12\xf4\x01\n\x15SetRegisteredModelTag\x12\x34.mlflow.ucmodelregistry.SetRegisteredModelTagRequest\x1a\x35.mlflow.ucmodelregistry.SetRegisteredModelTagResponse\"n\xf2\x86\x19j\n=\n\x04POST\x12//mlflow/unity-catalog/registered-models/set-tag\x1a\x04\x08\x02\x10\x00\x10\x03*\'(Unity Catalog) Set RegisteredModel Tag\x12\x85\x02\n\x18\x44\x65leteRegisteredModelTag\x12\x37.mlflow.ucmodelregistry.DeleteRegisteredModelTagRequest\x1a\x38.mlflow.ucmodelregistry.DeleteRegisteredModelTagResponse\"v\xf2\x86\x19r\nB\n\x06\x44\x45LETE\x12\x32/mlflow/unity-catalog/registered-models/delete-tag\x1a\x04\x08\x02\x10\x00\x10\x03**(Unity Catalog) Delete RegisteredModel Tag\x12\xe5\x01\n\x12SetModelVersionTag\x12\x31.mlflow.ucmodelregistry.SetModelVersionTagRequest\x1a\x32.mlflow.ucmodelregistry.SetModelVersionTagResponse\"h\xf2\x86\x19\x64\n:\n\x04POST\x12,/mlflow/unity-catalog/model-versions/set-tag\x1a\x04\x08\x02\x10\x00\x10\x03*$(Unity Catalog) Set ModelVersion Tag\x12\xf6\x01\n\x15\x44\x65leteModelVersionTag\x12\x34.mlflow.ucmodelregistry.DeleteModelVersionTagRequest\x1a\x35.mlflow.ucmodelregistry.DeleteModelVersionTagResponse\"p\xf2\x86\x19l\n?\n\x06\x44\x45LETE\x12//mlflow/unity-catalog/model-versions/delete-tag\x1a\x04\x08\x02\x10\x00\x10\x03*\'(Unity Catalog) Delete ModelVersion Tag\x12\xfe\x01\n\x17\x45mitModelVersionLineage\x12\x36.mlflow.ucmodelregistry.EmitModelVersionLineageRequest\x1a\x37.mlflow.ucmodelregistry.EmitModelVersionLineageResponse\"r\xf2\x86\x19n\n?\n\x04POST\x12\x31/mlflow/unity-catalog/model-versions/emit-lineage\x1a\x04\x08\x02\x10\x00\x10\x03*)(Unity Catalog) Emit ModelVersion Lineage\x12\xa3\x03\n.IsDatabricksSdkModelsArtifactRepositoryEnabled\x12M.mlflow.ucmodelregistry.IsDatabricksSdkModelsArtifactRepositoryEnabledRequest\x1aN.mlflow.ucmodelregistry.IsDatabricksSdkModelsArtifactRepositoryEnabledResponse\"\xd1\x01\xf2\x86\x19\xcc\x01\ni\n\x03GET\x12\\/mlflow/unity-catalog/registered-models:is-databricks-sdk-models-artifact-repository-enabled\x1a\x04\x08\x02\x10\x00\x10\x03*](Unity Catalog) Use DatabricksSdkModelsArtifactRepository in mlflow client for model registryB5\n(com.databricks.api.proto.ucmodelregistry\x90\x01\x01\xa0\x01\x01\xe2?\x02\x10\x01')



  _UCMODELREGISTRYSERVICE = DESCRIPTOR.services_by_name['UcModelRegistryService']
  if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b'\n(com.databricks.api.proto.ucmodelregistry\220\001\001\240\001\001\342?\002\020\001'
    _UCMODELREGISTRYSERVICE.methods_by_name['CreateRegisteredModel']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['CreateRegisteredModel']._serialized_options = b'\362\206\031h\n<\n\004POST\022./mlflow/unity-catalog/registered-models/create\032\004\010\002\020\000\020\003*&(Unity Catalog) Create RegisteredModel'
    _UCMODELREGISTRYSERVICE.methods_by_name['UpdateRegisteredModel']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['UpdateRegisteredModel']._serialized_options = b'\362\206\031i\n=\n\005PATCH\022./mlflow/unity-catalog/registered-models/update\032\004\010\002\020\000\020\003*&(Unity Catalog) Update RegisteredModel'
    _UCMODELREGISTRYSERVICE.methods_by_name['DeleteRegisteredModel']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['DeleteRegisteredModel']._serialized_options = b'\362\206\031j\n>\n\006DELETE\022./mlflow/unity-catalog/registered-models/delete\032\004\010\002\020\000\020\003*&(Unity Catalog) Delete RegisteredModel'
    _UCMODELREGISTRYSERVICE.methods_by_name['GetRegisteredModel']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['GetRegisteredModel']._serialized_options = b'\362\206\031a\n8\n\003GET\022+/mlflow/unity-catalog/registered-models/get\032\004\010\002\020\000\020\003*#(Unity Catalog) Get RegisteredModel'
    _UCMODELREGISTRYSERVICE.methods_by_name['SearchRegisteredModels']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['SearchRegisteredModels']._serialized_options = b'\362\206\031h\n;\n\003GET\022./mlflow/unity-catalog/registered-models/search\032\004\010\002\020\000\020\003*\'(Unity Catalog) Search RegisteredModels'
    _UCMODELREGISTRYSERVICE.methods_by_name['CreateModelVersion']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['CreateModelVersion']._serialized_options = b'\362\206\031b\n9\n\004POST\022+/mlflow/unity-catalog/model-versions/create\032\004\010\002\020\000\020\003*#(Unity Catalog) Create ModelVersion'
    _UCMODELREGISTRYSERVICE.methods_by_name['UpdateModelVersion']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['UpdateModelVersion']._serialized_options = b'\362\206\031c\n:\n\005PATCH\022+/mlflow/unity-catalog/model-versions/update\032\004\010\002\020\000\020\003*#(Unity Catalog) Update ModelVersion'
    _UCMODELREGISTRYSERVICE.methods_by_name['DeleteModelVersion']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['DeleteModelVersion']._serialized_options = b'\362\206\031d\n;\n\006DELETE\022+/mlflow/unity-catalog/model-versions/delete\032\004\010\002\020\000\020\003*#(Unity Catalog) Delete ModelVersion'
    _UCMODELREGISTRYSERVICE.methods_by_name['GetModelVersion']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['GetModelVersion']._serialized_options = b'\362\206\031[\n5\n\003GET\022(/mlflow/unity-catalog/model-versions/get\032\004\010\002\020\000\020\003* (Unity Catalog) Get ModelVersion'
    _UCMODELREGISTRYSERVICE.methods_by_name['SearchModelVersions']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['SearchModelVersions']._serialized_options = b'\362\206\031b\n8\n\003GET\022+/mlflow/unity-catalog/model-versions/search\032\004\010\002\020\000\020\003*$(Unity Catalog) Search ModelVersions'
    _UCMODELREGISTRYSERVICE.methods_by_name['GenerateTemporaryModelVersionCredentials']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['GenerateTemporaryModelVersionCredentials']._serialized_options = b'\362\206\031\223\001\nQ\n\004POST\022C/mlflow/unity-catalog/model-versions/generate-temporary-credentials\032\004\010\002\020\000\020\003*<(Unity Catalog) Generate Temporary Model Version Credentials'
    _UCMODELREGISTRYSERVICE.methods_by_name['GetModelVersionDownloadUri']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['GetModelVersionDownloadUri']._serialized_options = b'\362\206\031\203\001\nB\n\003GET\0225/mlflow/unity-catalog/model-versions/get-download-uri\032\004\010\002\020\000\020\003*;(Unity Catalog) Get Download URI For ModelVersion Artifacts'
    _UCMODELREGISTRYSERVICE.methods_by_name['FinalizeModelVersion']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['FinalizeModelVersion']._serialized_options = b'\362\206\031g\n;\n\004POST\022-/mlflow/unity-catalog/model-versions/finalize\032\004\010\002\020\000\020\003*&(Unity Catalog) Finalize Model Version'
    _UCMODELREGISTRYSERVICE.methods_by_name['SetRegisteredModelAlias']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['SetRegisteredModelAlias']._serialized_options = b'\362\206\031j\n;\n\004POST\022-/mlflow/unity-catalog/registered-models/alias\032\004\010\002\020\000\020\003*)(Unity Catalog) Set RegisteredModel Alias'
    _UCMODELREGISTRYSERVICE.methods_by_name['DeleteRegisteredModelAlias']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['DeleteRegisteredModelAlias']._serialized_options = b'\362\206\031o\n=\n\006DELETE\022-/mlflow/unity-catalog/registered-models/alias\032\004\010\002\020\000\020\003*,(Unity Catalog) Delete RegisteredModel Alias'
    _UCMODELREGISTRYSERVICE.methods_by_name['GetModelVersionByAlias']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['GetModelVersionByAlias']._serialized_options = b'\362\206\031i\n:\n\003GET\022-/mlflow/unity-catalog/registered-models/alias\032\004\010\002\020\000\020\003*)(Unity Catalog) Get ModelVersion By Alias'
    _UCMODELREGISTRYSERVICE.methods_by_name['SetRegisteredModelTag']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['SetRegisteredModelTag']._serialized_options = b'\362\206\031j\n=\n\004POST\022//mlflow/unity-catalog/registered-models/set-tag\032\004\010\002\020\000\020\003*\'(Unity Catalog) Set RegisteredModel Tag'
    _UCMODELREGISTRYSERVICE.methods_by_name['DeleteRegisteredModelTag']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['DeleteRegisteredModelTag']._serialized_options = b'\362\206\031r\nB\n\006DELETE\0222/mlflow/unity-catalog/registered-models/delete-tag\032\004\010\002\020\000\020\003**(Unity Catalog) Delete RegisteredModel Tag'
    _UCMODELREGISTRYSERVICE.methods_by_name['SetModelVersionTag']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['SetModelVersionTag']._serialized_options = b'\362\206\031d\n:\n\004POST\022,/mlflow/unity-catalog/model-versions/set-tag\032\004\010\002\020\000\020\003*$(Unity Catalog) Set ModelVersion Tag'
    _UCMODELREGISTRYSERVICE.methods_by_name['DeleteModelVersionTag']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['DeleteModelVersionTag']._serialized_options = b'\362\206\031l\n?\n\006DELETE\022//mlflow/unity-catalog/model-versions/delete-tag\032\004\010\002\020\000\020\003*\'(Unity Catalog) Delete ModelVersion Tag'
    _UCMODELREGISTRYSERVICE.methods_by_name['EmitModelVersionLineage']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['EmitModelVersionLineage']._serialized_options = b'\362\206\031n\n?\n\004POST\0221/mlflow/unity-catalog/model-versions/emit-lineage\032\004\010\002\020\000\020\003*)(Unity Catalog) Emit ModelVersion Lineage'
    _UCMODELREGISTRYSERVICE.methods_by_name['IsDatabricksSdkModelsArtifactRepositoryEnabled']._options = None
    _UCMODELREGISTRYSERVICE.methods_by_name['IsDatabricksSdkModelsArtifactRepositoryEnabled']._serialized_options = b'\362\206\031\314\001\ni\n\003GET\022\\/mlflow/unity-catalog/registered-models:is-databricks-sdk-models-artifact-repository-enabled\032\004\010\002\020\000\020\003*](Unity Catalog) Use DatabricksSdkModelsArtifactRepository in mlflow client for model registry'
    _UCMODELREGISTRYSERVICE._serialized_start=145
    _UCMODELREGISTRYSERVICE._serialized_end=5841
  UcModelRegistryService = service_reflection.GeneratedServiceType('UcModelRegistryService', (_service.Service,), dict(
    DESCRIPTOR = _UCMODELREGISTRYSERVICE,
    __module__ = 'databricks_uc_registry_service_pb2'
    ))

  UcModelRegistryService_Stub = service_reflection.GeneratedServiceStubType('UcModelRegistryService_Stub', (UcModelRegistryService,), dict(
    DESCRIPTOR = _UCMODELREGISTRYSERVICE,
    __module__ = 'databricks_uc_registry_service_pb2'
    ))


  # @@protoc_insertion_point(module_scope)




"""
This module provides a set of functions to manage the global tracer provider for MLflow tracing.

Every tracing operation in MLflow *MUST* be managed through this module, instead of directly
using the OpenTelemetry APIs. This is because MLflow needs to control the initialization of the
tracer provider and ensure that it won't interfere with the other external libraries that might
use OpenTelemetry e.g. PromptFlow, Snowpark.
"""

import contextvars
import functools
import json
import logging
import os
from contextlib import contextmanager
from typing import TYPE_CHECKING

from opentelemetry import context as context_api
from opentelemetry import trace
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import SpanProcessor, TracerProvider
from opentelemetry.sdk.trace.sampling import TraceIdRatioBased

import mlflow
from mlflow.entities.trace_location import (
    MlflowExperimentLocation,
    TraceLocationBase,
    UCSchemaLocation,
)
from mlflow.environment_variables import (
    MLFLOW_TRACE_ENABLE_OTLP_DUAL_EXPORT,
    MLFLOW_TRACE_SAMPLING_RATIO,
    MLFLOW_USE_DEFAULT_TRACER_PROVIDER,
)
from mlflow.exceptions import MlflowException, MlflowTracingException
from mlflow.tracing.config import reset_config
from mlflow.tracing.constant import SpanAttributeKey
from mlflow.tracing.destination import TraceDestination, UserTraceDestinationRegistry
from mlflow.tracing.utils.exception import raise_as_trace_exception
from mlflow.tracing.utils.once import Once
from mlflow.tracing.utils.otlp import (
    get_otlp_exporter,
    should_export_otlp_metrics,
    should_use_otlp_exporter,
)
from mlflow.tracing.utils.warning import suppress_warning
from mlflow.utils.databricks_utils import (
    is_in_databricks_model_serving_environment,
    is_mlflow_tracing_enabled_in_model_serving,
)

if TYPE_CHECKING:
    from mlflow.entities import Span


# A trace destination specified by the user via the `set_destination` function.
_MLFLOW_TRACE_USER_DESTINATION = UserTraceDestinationRegistry()


_logger = logging.getLogger(__name__)


class _TracerProviderWrapper:
    """
    A facade for the tracer provider.
    MLflow uses different tracer providers depending on the MLFLOW_USE_DEFAULT_TRACER_PROVIDER
    environment variable setting.
    1. Use an isolated tracer provider instance managed by MLflow. This is the default behavior such
       that MLflow does not break an environment where MLflow and OpenTelemetry SDK are used in
       different purposes.
    2. Use the global OpenTelemetry tracer provider singleton and traces created by MLflow and
        OpenTelemetry SDK will be exported to the same destination.
    """

    def __init__(self):
        self._isolated_tracer_provider = None
        self._isolated_tracer_provider_once = Once()

    @property
    def once(self) -> Once:
        if MLFLOW_USE_DEFAULT_TRACER_PROVIDER.get():
            return self._isolated_tracer_provider_once
        return trace._TRACER_PROVIDER_SET_ONCE

    def get(self) -> TracerProvider:
        if MLFLOW_USE_DEFAULT_TRACER_PROVIDER.get():
            return self._isolated_tracer_provider
        return trace.get_tracer_provider()

    def set(self, tracer_provider: TracerProvider):
        if MLFLOW_USE_DEFAULT_TRACER_PROVIDER.get():
            self._isolated_tracer_provider = tracer_provider
        else:
            # Bypass the once flag otherwise the update will be ignored.
            # We check the once flag inside `get_or_init_tracer`. For other cases, trace provider
            # should be forcibly updated.
            trace._TRACER_PROVIDER = tracer_provider

    def get_or_init_tracer(self, module_name: str) -> trace.Tracer:
        self.once.do_once(_initialize_tracer_provider)
        return self.get().get_tracer(module_name)

    def reset(self):
        if MLFLOW_USE_DEFAULT_TRACER_PROVIDER.get():
            self._isolated_tracer_provider = None
            self._isolated_tracer_provider_once._done = False
        else:
            trace._TRACER_PROVIDER = None
            trace._TRACER_PROVIDER_SET_ONCE._done = False


provider = _TracerProviderWrapper()


def start_span_in_context(name: str, experiment_id: str | None = None) -> trace.Span:
    """
    Start a new OpenTelemetry span in the current context.

    Note that this function doesn't set the started span as the active span in the context. To do
    that, the upstream also need to call `use_span()` function in the OpenTelemetry trace APIs.

    Args:
        name: The name of the span.
        experiment_id: The ID of the experiment to log the span to. If not specified, the span will
            be logged to the active experiment or explicitly set trace destination.

    Returns:
        The newly created OpenTelemetry span.
    """
    attributes = {}
    if experiment_id:
        attributes[SpanAttributeKey.EXPERIMENT_ID] = json.dumps(experiment_id)
    span = _get_tracer(__name__).start_span(name, attributes=attributes)

    if experiment_id and getattr(span, "_parent", None):
        _logger.warning(
            "The `experiment_id` parameter can only be used for root spans, but the span "
            f"`{name}` is not a root span. The specified value `{experiment_id}` will be ignored."
        )
        span._span.attributes.pop(SpanAttributeKey.EXPERIMENT_ID, None)
    return span


def start_detached_span(
    name: str,
    parent: trace.Span | None = None,
    experiment_id: str | None = None,
    start_time_ns: int | None = None,
) -> tuple[str, trace.Span] | None:
    """
    Start a new OpenTelemetry span that is not part of the current trace context, but with the
    explicit parent span ID if provided.

    Args:
        name: The name of the span.
        parent: The parent OpenTelemetry span. If not provided, the span will be created as a root
                span.
        experiment_id: The ID of the experiment. This is used to associate the span with a specific
            experiment in MLflow.
        start_time_ns: The start time of the span in nanoseconds.
            If not provided, the current timestamp is used.

    Returns:
        The newly created OpenTelemetry span.
    """
    tracer = _get_tracer(__name__)
    context = trace.set_span_in_context(parent) if parent else None
    attributes = {}

    # Set start time and experiment to attribute so we can pass it to the span processor
    if start_time_ns:
        attributes[SpanAttributeKey.START_TIME_NS] = json.dumps(start_time_ns)
    if experiment_id:
        attributes[SpanAttributeKey.EXPERIMENT_ID] = json.dumps(experiment_id)
    span = tracer.start_span(name, context=context, attributes=attributes, start_time=start_time_ns)

    if experiment_id and getattr(span, "_parent", None):
        _logger.warning(
            "The `experiment_id` parameter can only be used for root spans, but the span "
            f"`{name}` is not a root span. The specified value `{experiment_id}` will be ignored."
        )
        span._span.attributes.pop(SpanAttributeKey.EXPERIMENT_ID, None)
    return span


@contextmanager
def safe_set_span_in_context(span: "Span"):
    """
    A context manager that sets the given OpenTelemetry span as the active span in the current
    context.

    Args:
        span: An MLflow span object to set as the active span.

    Example:

    .. code-block:: python

        import mlflow


        with mlflow.start_span("my_span") as span:
            span.set_attribute("my_key", "my_value")

        # The span is automatically detached from the context when the context manager exits.
    """
    token = set_span_in_context(span)
    try:
        yield
    finally:
        detach_span_from_context(token)


def set_span_in_context(span: "Span") -> contextvars.Token:
    """
    Set the given OpenTelemetry span as the active span in the current context.

    Args:
        span: An MLflow span object to set as the active span.

    Returns:
        A token object that will be required when detaching the span from the context.
    """
    context = trace.set_span_in_context(span._span)
    token = context_api.attach(context)
    return token  # noqa: RET504


def detach_span_from_context(token: contextvars.Token):
    """
    Remove the active span from the current context.

    Args:
        token: The token returned by `_set_span_to_active` function.
    """
    context_api.detach(token)


def set_destination(destination: TraceLocationBase, *, context_local: bool = False):
    """
    Set a custom span location to which MLflow will export the traces.

    A destination specified by this function will take precedence over
    other configurations, such as tracking URI, OTLP environment variables.

    Args:
        destination: A trace location object that specifies the location of the trace data.
            Currently, the following locations are supported:

            - :py:class:`~mlflow.entities.trace_location.MlflowExperimentLocation`: Logs traces to
                an MLflow experiment.
            - :py:class:`~mlflow.entities.trace_location.UCSchemaLocation`: Logs traces to a
                Databricks Unity Catalog schema. Only available in Databricks.

        context_local: If False (default), the destination is set globally. If True, the destination
            is isolated per async task or thread, providing isolation in concurrent applications.

    Example:

        **Logging traces to MLflow Experiment:**

        .. code-block:: python

            from mlflow.entities.trace_location import MlflowExperimentLocation

            mlflow.tracing.set_destination(MlflowExperimentLocation(experiment_id="123"))

        Note: This has the same effect as setting the active MLflow experiment via the
        ``MLFLOW_EXPERIMENT_ID`` environment variable or the ``mlflow.set_experiment`` API,
        but with narrower scope.

        **Logging traces to Databricks Unity Catalog:**

        .. code-block:: python

            from mlflow.entities.trace_location import UCSchemaLocation

            mlflow.tracing.set_destination(
                UCSchemaLocation(catalog_name="catalog", schema_name="schema")
            )

        **Isolate the destination between async tasks or threads:**

        .. code-block:: python

            from mlflow.tracing.destination import Databricks

            mlflow.tracing.set_destination(
                MlflowExperimentLocation(experiment_id="123"),
                context_local=True,
            )

        The destination set with the ``context_local`` flag will only be effective within the
        current async task or thread. This is particularly useful when you want to send traces
        to different destinations from a multi-tenant application.

        ** Reset the destination:**

        .. code-block:: python

            mlflow.tracing.reset()

    """
    if isinstance(destination, TraceDestination):
        # NB: Deprecation warnings are issued in the constructor of the destination classes
        # so we don't need to issue a warning here.
        destination = destination.to_location()

    if not isinstance(destination, TraceLocationBase):
        raise MlflowException.invalid_parameter_value(
            f"Invalid destination type: {type(destination)}. "
            "The destination must be an instance of TraceLocation."
        )

    if isinstance(destination, UCSchemaLocation) and (
        mlflow.get_tracking_uri() is None or not mlflow.get_tracking_uri().startswith("databricks")
    ):
        mlflow.set_tracking_uri("databricks")
        _logger.info(
            "Automatically setting the tracking URI to `databricks` "
            "because the tracing destination is set to Databricks."
        )

    _MLFLOW_TRACE_USER_DESTINATION.set(destination, context_local=context_local)
    _initialize_tracer_provider()


def _get_tracer(module_name: str) -> trace.Tracer:
    """
    Get a tracer instance for the given module name.

    If the tracer provider is not initialized, this function will initialize the tracer provider.
    Other simultaneous calls to this function will block until the initialization is done.
    """
    return provider.get_or_init_tracer(module_name)


def _get_trace_exporter():
    """
    Get the exporter instance that is used by the current tracer provider.
    """
    if tracer_provider := provider.get():
        processors = tracer_provider._active_span_processor._span_processors
        # There should be only one processor used for MLflow tracing
        processor = processors[0]
        return processor.span_exporter


def _initialize_tracer_provider(disabled=False):
    """
    Instantiate a tracer provider and set it as the global tracer provider.

    Note that this function ALWAYS updates the global tracer provider, regardless of the current
    state. It is the caller's responsibility to ensure that the tracer provider is initialized
    only once, and update the _INITIALIZED flag accordingly.
    """
    processors = _get_span_processors(disabled=disabled)
    if not processors:
        provider.set(trace.NoOpTracerProvider())
        return

    # Demote the "Failed to detach context" log raised by the OpenTelemetry logger to DEBUG
    # level so that it does not show up in the user's console. This warning may indicate
    # some incorrect context handling, but in many cases just false positive that does not
    # cause any issue in the generated trace.
    # Note that we need to apply it permanently rather than just the scope of prediction call,
    # because the exception can happen for streaming case, where the error log might be
    # generated when the iterator is consumed and we don't know when it will happen.
    suppress_warning("opentelemetry.context", "Failed to detach context")

    # These Otel warnings are occasionally raised in a valid case, e.g. we timeout a trace
    # but some spans are still active. We suppress them because they are not actionable.
    suppress_warning("opentelemetry.sdk.trace", "Setting attribute on ended span")
    suppress_warning("opentelemetry.sdk.trace", "Calling end() on an ended span")

    # NB: If otel resource env vars are set explicitly, don't create an empty resource
    # so that they are propagated to otel spans.
    otel_service_name = os.getenv("OTEL_SERVICE_NAME")
    otel_resource_attributes = os.getenv("OTEL_RESOURCE_ATTRIBUTES")
    resource = None
    if not otel_service_name and not otel_resource_attributes:
        # Setting an empty resource to avoid triggering resource aggregation, which causes
        # an issue in LiteLLM tracing: https://github.com/mlflow/mlflow/issues/16296
        # Add telemetry resource: https://opentelemetry.io/docs/specs/semconv/resource/#telemetry-sdk
        resource = Resource(
            {
                "telemetry.sdk.language": "python",
                "telemetry.sdk.name": "mlflow",
                "telemetry.sdk.version": mlflow.__version__,
            }
        )
    tracer_provider = TracerProvider(resource=resource, sampler=_get_trace_sampler())
    for processor in processors:
        tracer_provider.add_span_processor(processor)

    provider.set(tracer_provider)


def _get_trace_sampler() -> TraceIdRatioBased | None:
    """
    Get the sampler configuration based on environment variable.

    Returns:
        TraceIdRatioBased sampler or None for default sampling.
    """
    sampling_ratio = MLFLOW_TRACE_SAMPLING_RATIO.get()
    if sampling_ratio is not None:
        if not (0.0 <= sampling_ratio <= 1.0):
            _logger.warning(
                f"{MLFLOW_TRACE_SAMPLING_RATIO} must be between 0.0 and 1.0, got {sampling_ratio}. "
                "Ignoring the invalid value and using default sampling (1.0)."
            )
            return None
        return TraceIdRatioBased(sampling_ratio)
    return None


def _get_span_processors(disabled: bool = False) -> list[SpanProcessor]:
    """
    Get the list of span processors based on configuration.

    Args:
        disabled: If True, returns an empty list of processors because tracing is disabled.

    Returns:
        List of span processors to be added to the TracerProvider.
    """
    if disabled:
        return []

    processors = []

    # TODO: Update this logic to pluggable registry where
    #  1. Partners can implement span processor/exporter and destination class.
    #  2. They can register their implementation to the registry via entry points.
    #  3. MLflow will pick the implementation based on given destination id.
    if trace_destination := _MLFLOW_TRACE_USER_DESTINATION.get():
        # In PrPr, users must set the destination to UCSchemaLocation to export traces to UC
        if isinstance(trace_destination, UCSchemaLocation):
            from mlflow.tracing.export.uc_table import DatabricksUCTableSpanExporter
            from mlflow.tracing.processor.uc_table import DatabricksUCTableSpanProcessor

            exporter = DatabricksUCTableSpanExporter(tracking_uri=mlflow.get_tracking_uri())
            processor = DatabricksUCTableSpanProcessor(span_exporter=exporter)
            processors.append(processor)

        elif isinstance(trace_destination, MlflowExperimentLocation):
            if is_in_databricks_model_serving_environment():
                _logger.info(
                    "Traces will be sent to the destination set by "
                    "`mlflow.tracing.set_destination` API. To enable saving traces to "
                    "both MLflow experiment and inference table, remove this API call "
                    "from your model and set `MLFLOW_EXPERIMENT_ID` env var instead."
                )
            processor = _get_mlflow_span_processor(tracking_uri=mlflow.get_tracking_uri())
            processors.append(processor)

        # Trace destination has highest precedence; definitely ignore defaults
        # Ignore OTLP (unless dual export is set and we should use OTLP)
        if not (should_use_otlp_exporter() and MLFLOW_TRACE_ENABLE_OTLP_DUAL_EXPORT.get()):
            return processors

    # If no explicit trace destination OR we passed the dual exporter check, honor OTLP
    # configuration.
    if should_use_otlp_exporter():
        from mlflow.tracing.processor.otel import OtelSpanProcessor

        exporter = get_otlp_exporter()
        otel_processor = OtelSpanProcessor(
            span_exporter=exporter,
            # Only export metrics from the Otel processor if dual export is not enabled.
            # Otherwise, both Otel and MLflow processors will export metrics, causing
            # duplication.
            export_metrics=should_export_otlp_metrics()
            and not MLFLOW_TRACE_ENABLE_OTLP_DUAL_EXPORT.get(),
        )
        processors.append(otel_processor)

        # We have now added the OTLP processor.
        # If dual export is not set, return.
        # If dual export is set AND we have already added a set_destination processor, return.
        # If dual export is set but no set_destination processor added, skip return and go
        # to default processing to catch the default processor, if present.
        if (not MLFLOW_TRACE_ENABLE_OTLP_DUAL_EXPORT.get()) or any(
            not isinstance(p, OtelSpanProcessor) for p in processors
        ):
            return processors
    # Finally, default MLflow-based processors (inference table in serving, else tracking URI).
    if is_in_databricks_model_serving_environment():
        if not is_mlflow_tracing_enabled_in_model_serving():
            # May still return an OTLP processor from above, or nothing at all.
            return processors

        from mlflow.tracing.export.inference_table import InferenceTableSpanExporter
        from mlflow.tracing.processor.inference_table import InferenceTableSpanProcessor

        exporter = InferenceTableSpanExporter()
        processor = InferenceTableSpanProcessor(exporter)
        processors.append(processor)
    else:
        processor = _get_mlflow_span_processor(tracking_uri=mlflow.get_tracking_uri())
        processors.append(processor)

    return processors


def _get_mlflow_span_processor(tracking_uri: str):
    """
    Get the MLflow span processor instance that is used by the current tracer provider.
    """
    # Databricks and SQL backends support V3 traces
    from mlflow.tracing.export.mlflow_v3 import MlflowV3SpanExporter
    from mlflow.tracing.processor.mlflow_v3 import MlflowV3SpanProcessor

    exporter = MlflowV3SpanExporter(tracking_uri=tracking_uri)
    return MlflowV3SpanProcessor(
        span_exporter=exporter,
        export_metrics=should_export_otlp_metrics(),
    )


@raise_as_trace_exception
def disable():
    """
    Disable tracing.

    .. note::

        This function sets up `OpenTelemetry` to use
        `NoOpTracerProvider <https://github.com/open-telemetry/opentelemetry-python/blob/4febd337b019ea013ccaab74893bd9883eb59000/opentelemetry-api/src/opentelemetry/trace/__init__.py#L222>`_
        and effectively disables all tracing operations.

    Example:

    .. code-block:: python
        :test:

        import mlflow


        @mlflow.trace
        def f():
            return 0


        # Tracing is enabled by default
        f()
        assert len(mlflow.search_traces()) == 1

        # Disable tracing
        mlflow.tracing.disable()
        f()
        assert len(mlflow.search_traces()) == 1

    """
    if not is_tracing_enabled():
        return

    _initialize_tracer_provider(disabled=True)
    provider.once._done = True


@raise_as_trace_exception
def enable():
    """
    Enable tracing.

    Example:

    .. code-block:: python
        :test:

        import mlflow


        @mlflow.trace
        def f():
            return 0


        # Tracing is enabled by default
        f()
        assert len(mlflow.search_traces()) == 1

        # Disable tracing
        mlflow.tracing.disable()
        f()
        assert len(mlflow.search_traces()) == 1

        # Re-enable tracing
        mlflow.tracing.enable()
        f()
        assert len(mlflow.search_traces()) == 2

    """
    if is_tracing_enabled() and provider.once._done:
        _logger.info("Tracing is already enabled")
        return

    _initialize_tracer_provider()
    provider.once._done = True


def trace_disabled(f):
    """
    A decorator that temporarily disables tracing for the duration of the decorated function.

    .. code-block:: python

        @trace_disabled
        def f():
            with mlflow.start_span("my_span") as span:
                span.set_attribute("my_key", "my_value")

            return


        # This function will not generate any trace
        f()

    :meta private:
    """

    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        is_func_called = False
        result = None
        try:
            if is_tracing_enabled():
                disable()
                try:
                    is_func_called = True
                    result = f(*args, **kwargs)
                finally:
                    enable()
            else:
                is_func_called = True
                result = f(*args, **kwargs)
        # We should only catch the exception from disable() and enable()
        # and let other exceptions propagate.
        except MlflowTracingException as e:
            _logger.warning(
                f"An error occurred while disabling or re-enabling tracing: {e} "
                "The original function will still be executed, but the tracing "
                "state may not be as expected. For full traceback, set "
                "logging level to debug.",
                exc_info=_logger.isEnabledFor(logging.DEBUG),
            )
            # If the exception is raised before the original function
            # is called, we should call the original function
            if not is_func_called:
                result = f(*args, **kwargs)

        return result

    return wrapper


def reset():
    """
    Reset the flags that indicates whether the MLflow tracer provider has been initialized.
    This ensures that the tracer provider is re-initialized when next tracing
    operation is performed.
    """
    # Set NoOp tracer provider to reset the global tracer to the initial state.
    _initialize_tracer_provider(disabled=True)
    # Flip the "once" flag to False so that
    # the next tracing operation will re-initialize the provider.
    provider.reset()

    # Reset the custom destination set by the user
    _MLFLOW_TRACE_USER_DESTINATION.reset()

    # Reset the tracing configuration to defaults
    reset_config()


@raise_as_trace_exception
def is_tracing_enabled() -> bool:
    """
    Check if tracing is enabled based on whether the global tracer
    is instantiated or not.

    Trace is considered as "enabled" if the followings
    1. The default state (before any tracing operation)
    2. The tracer is not either ProxyTracer or NoOpTracer
    """
    if not provider.once._done:
        return True

    tracer = _get_tracer(__name__)
    # Occasionally ProxyTracer instance wraps the actual tracer
    if isinstance(tracer, trace.ProxyTracer):
        tracer = tracer._tracer
    return not isinstance(tracer, trace.NoOpTracer)



"""
FastAPI application wrapper for MLflow server.

This module provides a FastAPI application that wraps the existing Flask application
using WSGIMiddleware to maintain 100% API compatibility while enabling future migration
to FastAPI endpoints.
"""

from fastapi import FastAPI
from fastapi.middleware.wsgi import WSGIMiddleware
from flask import Flask

from mlflow.server import app as flask_app
from mlflow.server.fastapi_security import init_fastapi_security
from mlflow.server.job_api import job_api_router
from mlflow.server.otel_api import otel_router
from mlflow.version import VERSION


def create_fastapi_app(flask_app: Flask = flask_app):
    """
    Create a FastAPI application that wraps the existing Flask app.

    Returns:
        FastAPI application instance with the Flask app mounted via WSGIMiddleware.
    """
    # Create FastAPI app with metadata
    fastapi_app = FastAPI(
        title="MLflow Tracking Server",
        description="MLflow Tracking Server API",
        version=VERSION,
        # TODO: Enable API documentation when we have native FastAPI endpoints
        # For now, disable docs since we only have Flask routes via WSGI
        docs_url=None,
        redoc_url=None,
        openapi_url=None,
    )

    # Initialize security middleware BEFORE adding routes
    init_fastapi_security(fastapi_app)

    # Include OpenTelemetry API router BEFORE mounting Flask app
    # This ensures FastAPI routes take precedence over the catch-all Flask mount
    fastapi_app.include_router(otel_router)

    fastapi_app.include_router(job_api_router)

    # Mount the entire Flask application at the root path
    # This ensures compatibility with existing APIs
    # NOTE: This must come AFTER include_router to avoid Flask catching all requests
    fastapi_app.mount("/", WSGIMiddleware(flask_app))

    return fastapi_app


# Create the app instance that can be used by ASGI servers
app = create_fastapi_app()



from mlflow.dspy.autolog import autolog
from mlflow.version import IS_TRACING_SDK_ONLY

__all__ = ["autolog"]

# Import model logging APIs only if mlflow skinny or full package is installed,
# i.e., skip if only mlflow-tracing package is installed.
if not IS_TRACING_SDK_ONLY:
    from mlflow.dspy.load import _load_pyfunc, load_model
    from mlflow.dspy.save import log_model, save_model

    __all__ += [
        "save_model",
        "log_model",
        "load_model",
        "_load_pyfunc",
    ]



from __future__ import annotations

import json
from dataclasses import dataclass
from typing import Any

from google.protobuf.json_format import MessageToDict

from mlflow.entities._mlflow_object import _MlflowObject
from mlflow.entities.dataset_record_source import DatasetRecordSource, DatasetRecordSourceType
from mlflow.protos.datasets_pb2 import DatasetRecord as ProtoDatasetRecord
from mlflow.protos.datasets_pb2 import DatasetRecordSource as ProtoDatasetRecordSource

# Reserved key for wrapping non-dict outputs when storing in SQL database
DATASET_RECORD_WRAPPED_OUTPUT_KEY = "mlflow_wrapped"


@dataclass
class DatasetRecord(_MlflowObject):
    """Represents a single record in an evaluation dataset.

    A DatasetRecord contains the input data, expected outputs (ground truth),
    and metadata for a single evaluation example. Records are immutable once
    created and are uniquely identified by their dataset_record_id.
    """

    dataset_id: str
    inputs: dict[str, Any]
    dataset_record_id: str
    created_time: int
    last_update_time: int
    outputs: dict[str, Any] | None = None
    expectations: dict[str, Any] | None = None
    tags: dict[str, str] | None = None
    source: DatasetRecordSource | None = None
    source_id: str | None = None
    source_type: str | None = None
    created_by: str | None = None
    last_updated_by: str | None = None

    def __post_init__(self):
        if self.inputs is None:
            raise ValueError("inputs must be provided")

        if self.tags is None:
            self.tags = {}

        if self.source and isinstance(self.source, DatasetRecordSource):
            if not self.source_id:
                if self.source.source_type == DatasetRecordSourceType.TRACE:
                    self.source_id = self.source.source_data.get("trace_id")
                else:
                    self.source_id = self.source.source_data.get("source_id")
            if not self.source_type:
                self.source_type = self.source.source_type.value

    def to_proto(self) -> ProtoDatasetRecord:
        proto = ProtoDatasetRecord()

        proto.dataset_record_id = self.dataset_record_id
        proto.dataset_id = self.dataset_id
        proto.inputs = json.dumps(self.inputs)
        proto.created_time = self.created_time
        proto.last_update_time = self.last_update_time
        if self.outputs is not None:
            proto.outputs = json.dumps(self.outputs)
        if self.expectations is not None:
            proto.expectations = json.dumps(self.expectations)
        if self.tags is not None:
            proto.tags = json.dumps(self.tags)
        if self.source is not None:
            proto.source = json.dumps(self.source.to_dict())
        if self.source_id is not None:
            proto.source_id = self.source_id
        if self.source_type is not None:
            proto.source_type = ProtoDatasetRecordSource.SourceType.Value(self.source_type)
        if self.created_by is not None:
            proto.created_by = self.created_by
        if self.last_updated_by is not None:
            proto.last_updated_by = self.last_updated_by

        return proto

    @classmethod
    def from_proto(cls, proto: ProtoDatasetRecord) -> "DatasetRecord":
        inputs = json.loads(proto.inputs) if proto.HasField("inputs") else {}
        outputs = json.loads(proto.outputs) if proto.HasField("outputs") else None
        expectations = json.loads(proto.expectations) if proto.HasField("expectations") else None
        tags = json.loads(proto.tags) if proto.HasField("tags") else None

        source = None
        if proto.HasField("source"):
            source_dict = json.loads(proto.source)
            source = DatasetRecordSource.from_dict(source_dict)

        return cls(
            dataset_id=proto.dataset_id,
            inputs=inputs,
            dataset_record_id=proto.dataset_record_id,
            created_time=proto.created_time,
            last_update_time=proto.last_update_time,
            outputs=outputs,
            expectations=expectations,
            tags=tags,
            source=source,
            source_id=proto.source_id if proto.HasField("source_id") else None,
            source_type=DatasetRecordSourceType.from_proto(proto.source_type)
            if proto.HasField("source_type")
            else None,
            created_by=proto.created_by if proto.HasField("created_by") else None,
            last_updated_by=proto.last_updated_by if proto.HasField("last_updated_by") else None,
        )

    def to_dict(self) -> dict[str, Any]:
        d = MessageToDict(
            self.to_proto(),
            preserving_proto_field_name=True,
        )
        d["inputs"] = json.loads(d["inputs"])
        if "outputs" in d:
            d["outputs"] = json.loads(d["outputs"])
        if "expectations" in d:
            d["expectations"] = json.loads(d["expectations"])
        if "tags" in d:
            d["tags"] = json.loads(d["tags"])
        if "source" in d:
            d["source"] = json.loads(d["source"])
        d["created_time"] = self.created_time
        d["last_update_time"] = self.last_update_time
        return d

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> "DatasetRecord":
        # Validate required fields
        if "dataset_id" not in data:
            raise ValueError("dataset_id is required")
        if "dataset_record_id" not in data:
            raise ValueError("dataset_record_id is required")
        if "inputs" not in data:
            raise ValueError("inputs is required")
        if "created_time" not in data:
            raise ValueError("created_time is required")
        if "last_update_time" not in data:
            raise ValueError("last_update_time is required")

        source = None
        if data.get("source"):
            source = DatasetRecordSource.from_dict(data["source"])

        return cls(
            dataset_id=data["dataset_id"],
            inputs=data["inputs"],
            dataset_record_id=data["dataset_record_id"],
            created_time=data["created_time"],
            last_update_time=data["last_update_time"],
            outputs=data.get("outputs"),
            expectations=data.get("expectations"),
            tags=data.get("tags"),
            source=source,
            source_id=data.get("source_id"),
            source_type=data.get("source_type"),
            created_by=data.get("created_by"),
            last_updated_by=data.get("last_updated_by"),
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, DatasetRecord):
            return False
        return (
            self.dataset_record_id == other.dataset_record_id
            and self.dataset_id == other.dataset_id
            and self.inputs == other.inputs
            and self.outputs == other.outputs
            and self.expectations == other.expectations
            and self.tags == other.tags
            and self.source == other.source
            and self.source_id == other.source_id
            and self.source_type == other.source_type
        )



from typing import Any, Literal, get_args, get_origin

from mlflow.genai.judges.base import Judge
from mlflow.genai.judges.instructions_judge import InstructionsJudge
from mlflow.telemetry.events import MakeJudgeEvent
from mlflow.telemetry.track import record_usage_event
from mlflow.utils.annotations import experimental


def _validate_feedback_value_type(feedback_value_type: Any) -> None:
    """
    Validate that feedback_value_type is one of the supported types for serialization.

    Supported types match FeedbackValueType:
    - PbValueType: int, float, str, bool
    - Literal types with PbValueType values
    - dict[str, PbValueType]
    - list[PbValueType]
    """

    from mlflow.entities.assessment import PbValueType

    # Check for basic PbValueType (float, int, str, bool)
    pb_value_types = get_args(PbValueType)
    if feedback_value_type in pb_value_types:
        return

    # Check for Literal type
    origin = get_origin(feedback_value_type)
    if origin is Literal:
        # Validate that all literal values are of PbValueType
        literal_values = get_args(feedback_value_type)
        for value in literal_values:
            if not isinstance(value, pb_value_types):
                from mlflow.exceptions import MlflowException

                raise MlflowException.invalid_parameter_value(
                    "The `feedback_value_type` argument does not support a Literal type"
                    f"with non-primitive types, but got {type(value).__name__}. "
                    f"Literal values must be str, int, float, or bool."
                )
        return

    # Check for dict[str, PbValueType]
    if origin is dict:
        args = get_args(feedback_value_type)
        if len(args) == 2:
            key_type, value_type = args
            # Key must be str
            if key_type != str:
                from mlflow.exceptions import MlflowException

                raise MlflowException.invalid_parameter_value(
                    f"dict key type must be str, got {key_type}"
                )
            # Value must be a PbValueType
            if value_type not in pb_value_types:
                from mlflow.exceptions import MlflowException

                raise MlflowException.invalid_parameter_value(
                    "The `feedback_value_type` argument does not support a dict type"
                    f"with non-primitive values, but got {value_type.__name__}"
                )
            return

    # Check for list[PbValueType]
    if origin is list:
        args = get_args(feedback_value_type)
        if len(args) == 1:
            element_type = args[0]
            # Element must be a PbValueType
            if element_type not in pb_value_types:
                from mlflow.exceptions import MlflowException

                raise MlflowException.invalid_parameter_value(
                    "The `feedback_value_type` argument does not support a list type"
                    f"with non-primitive values, but got {element_type.__name__}"
                )
            return

    # If we get here, it's an unsupported type
    from mlflow.exceptions import MlflowException

    raise MlflowException.invalid_parameter_value(
        f"Unsupported feedback_value_type: {feedback_value_type}. "
        f"Supported types (FeedbackValueType): str, int, float, bool, Literal[...], "
        f"as well as a dict and list of these types. "
        f"Pydantic BaseModel types are not supported."
    )


@experimental(version="3.4.0")
@record_usage_event(MakeJudgeEvent)
def make_judge(
    name: str,
    instructions: str,
    model: str | None = None,
    description: str | None = None,
    feedback_value_type: Any = None,
    inference_params: dict[str, Any] | None = None,
) -> Judge:
    """

    .. note::
        As of MLflow 3.4.0, this function is deprecated in favor of `mlflow.genai.make_judge`
        and may be removed in a future version.

    Create a custom MLflow judge instance.

    Args:
        name: The name of the judge
        instructions: Natural language instructions for evaluation. Must contain at least one
                      template variable: {{ inputs }}, {{ outputs }}, {{ expectations }},
                      {{ conversation }}, or {{ trace }} to reference evaluation data. Custom
                      variables are not supported.
                      Note: {{ conversation }} can only coexist with {{ expectations }}.
                      It cannot be used together with {{ inputs }}, {{ outputs }}, or {{ trace }}.
        model: The model identifier to use for evaluation (e.g., "openai:/gpt-4")
        description: A description of what the judge evaluates
        feedback_value_type: Type specification for the 'value' field in the Feedback
                        object. The judge will use structured outputs to enforce this type.
                        If unspecified, the feedback value type is determined by the judge.
                        It is recommended to explicitly specify the type.

                        Supported types (matching FeedbackValueType):

                        - int: Integer ratings (e.g., 1-5 scale)
                        - float: Floating point scores (e.g., 0.0-1.0)
                        - str: Text responses
                        - bool: Yes/no evaluations
                        - Literal[values]: Enum-like choices (e.g., Literal["good", "bad"])
                        - dict[str, int | float | str | bool]: Dictionary with string keys and
                          int, float, str, or bool values.
                        - list[int | float | str | bool]: List of int, float, str, or bool values

                        Note: Pydantic BaseModel types are not supported.
        inference_params: Optional dictionary of inference parameters to pass to the model
                        (e.g., temperature, top_p, max_tokens). These parameters allow
                        fine-grained control over the model's behavior during evaluation.
                        For example, setting a lower temperature can produce more
                        deterministic and reproducible evaluation results.

    Returns:
        An InstructionsJudge instance configured with the provided parameters

    Example:
        .. code-block:: python

            import mlflow
            from mlflow.genai.judges import make_judge
            from typing import Literal

            # Create a judge that evaluates response quality using template variables
            quality_judge = make_judge(
                name="response_quality",
                instructions=(
                    "Evaluate if the response in {{ outputs }} correctly answers "
                    "the question in {{ inputs }}. The response should be accurate, "
                    "complete, and professional."
                ),
                model="openai:/gpt-4",
                feedback_value_type=Literal["yes", "no"],
            )

            # Evaluate a response
            result = quality_judge(
                inputs={"question": "What is machine learning?"},
                outputs="ML is basically when computers learn stuff on their own",
            )

            # Create a judge that compares against expectations
            correctness_judge = make_judge(
                name="correctness",
                instructions=(
                    "Compare the {{ outputs }} against the {{ expectations }}. "
                    "Rate how well they match on a scale of 1-5."
                ),
                model="openai:/gpt-4",
                feedback_value_type=int,
            )

            # Evaluate with expectations (must be dictionaries)
            result = correctness_judge(
                inputs={"question": "What is the capital of France?"},
                outputs={"answer": "The capital of France is Paris."},
                expectations={"expected_answer": "Paris"},
            )

            # Create a judge that evaluates based on trace context
            trace_judge = make_judge(
                name="trace_quality",
                instructions="Evaluate the overall quality of the {{ trace }} execution.",
                model="openai:/gpt-4",
                feedback_value_type=Literal["good", "needs_improvement"],
            )

            # Use with search_traces() - evaluate each trace
            traces = mlflow.search_traces(experiment_ids=["1"], return_type="list")
            for trace in traces:
                feedback = trace_judge(trace=trace)
                print(f"Trace {trace.info.trace_id}: {feedback.value} - {feedback.rationale}")

            # Create a multi-turn judge that detects user frustration
            frustration_judge = make_judge(
                name="user_frustration",
                instructions=(
                    "Analyze the {{ conversation }} to detect signs of user frustration. "
                    "Look for indicators such as repeated questions, negative language, "
                    "or expressions of dissatisfaction."
                ),
                model="openai:/gpt-4",
                feedback_value_type=Literal["frustrated", "not frustrated"],
            )

            # Evaluate a multi-turn conversation using session traces
            session = mlflow.search_traces(
                experiment_ids=["1"],
                filter_string="metadata.`mlflow.trace.session` = 'session_123'",
                return_type="list",
            )
            result = frustration_judge(session=session)

            # Align a judge with human feedback
            aligned_judge = quality_judge.align(traces)

            # To see detailed optimization output during alignment, enable DEBUG logging:
            # import logging
            # logging.getLogger("mlflow.genai.judges.optimizers.simba").setLevel(logging.DEBUG)
    """
    # Default feedback_value_type to str if not specified (consistent with MLflow <= 3.5.x)
    # TODO: Implement logic to allow the LLM to choose the appropriate value type if not specified
    if feedback_value_type is None:
        feedback_value_type = str

    _validate_feedback_value_type(feedback_value_type)

    return InstructionsJudge(
        name=name,
        instructions=instructions,
        model=model,
        description=description,
        feedback_value_type=feedback_value_type,
        inference_params=inference_params,
    )



from pydantic import ConfigDict

from mlflow.gateway.base_models import ResponseModel
from mlflow.gateway.config import EndpointModelInfo, Limit


class Endpoint(ResponseModel):
    name: str
    endpoint_type: str
    model: EndpointModelInfo
    endpoint_url: str
    limit: Limit | None

    model_config = ConfigDict(
        json_schema_extra={
            "example": {
                "name": "openai-completions",
                "endpoint_type": "llm/v1/completions",
                "model": {
                    "name": "gpt-4o-mini",
                    "provider": "openai",
                },
                "endpoint_url": "/endpoints/completions/invocations",
                "limit": {"calls": 1, "key": None, "renewal_period": "minute"},
            }
        }
    )



from typing import Any

from mlflow.data.dataset_source import DatasetSource


class EvaluationDatasetSource(DatasetSource):
    """
    Represents the source of an evaluation dataset stored in MLflow's tracking store.
    """

    def __init__(self, dataset_id: str):
        """
        Args:
            dataset_id: The ID of the evaluation dataset.
        """
        self._dataset_id = dataset_id

    @staticmethod
    def _get_source_type() -> str:
        return "mlflow_evaluation_dataset"

    def load(self) -> Any:
        """
        Loads the evaluation dataset from the tracking store using current tracking URI.

        Returns:
            The EvaluationDataset entity.
        """
        from mlflow.tracking._tracking_service.utils import _get_store

        store = _get_store()
        return store.get_evaluation_dataset(self._dataset_id)

    @staticmethod
    def _can_resolve(raw_source: Any) -> bool:
        """
        Determines if the raw source is an evaluation dataset ID.
        """
        if isinstance(raw_source, str):
            return raw_source.startswith("d-") and len(raw_source) == 34
        return False

    @classmethod
    def _resolve(cls, raw_source: Any) -> "EvaluationDatasetSource":
        """
        Creates an EvaluationDatasetSource from a dataset ID.
        """
        if not cls._can_resolve(raw_source):
            raise ValueError(f"Cannot resolve {raw_source} as an evaluation dataset ID")

        return cls(dataset_id=raw_source)

    def to_dict(self) -> dict[str, Any]:
        return {
            "dataset_id": self._dataset_id,
        }

    @classmethod
    def from_dict(cls, source_dict: dict[Any, Any]) -> "EvaluationDatasetSource":
        return cls(
            dataset_id=source_dict["dataset_id"],
        )



import logging
import os
import subprocess
from contextvars import ContextVar

from mlflow.tracking.fluent import _set_active_model
from mlflow.utils.annotations import experimental

# Context-isolated storage for request headers
# ensuring thread-safe access across async execution contexts
_request_headers: ContextVar[dict[str, str]] = ContextVar[dict[str, str]](
    "request_headers", default={}
)

logger = logging.getLogger(__name__)


@experimental(version="3.6.0")
def set_request_headers(headers: dict[str, str]) -> None:
    """Set request headers in the current context (called by server)"""
    _request_headers.set(headers)


@experimental(version="3.6.0")
def get_request_headers() -> dict[str, str]:
    """Get all request headers from the current context"""
    return _request_headers.get()


@experimental(version="3.6.0")
def setup_mlflow_git_based_version_tracking() -> None:
    """Initialize MLflow tracking and set active model with git-based version tracking."""
    # in a Databricks App, the app name is set in the environment variable DATABRICKS_APP_NAME
    # in local development, we use a fallback app name
    app_name = os.getenv("DATABRICKS_APP_NAME", "local")

    # Get current git commit hash for versioning
    try:
        git_commit = (
            subprocess.check_output(["git", "rev-parse", "HEAD"]).decode("ascii").strip()[:8]
        )
        version_identifier = f"git-{git_commit}"
    except subprocess.CalledProcessError:
        version_identifier = "no-git"
    logged_model_name = f"{app_name}-{version_identifier}"

    # Set the active model context
    active_model_info = _set_active_model(name=logged_model_name)
    logger.info(
        f"Active LoggedModel: '{active_model_info.name}', Model ID: '{active_model_info.model_id}'"
    )



# NB: User-facing name for the conversation completeness assessment.
CONVERSATION_COMPLETENESS_ASSESSMENT_NAME = "conversation_completeness"

CONVERSATION_COMPLETENESS_PROMPT = """\
Consider the following conversation history between a user and an assistant.
Your task is to output exactly one label: "yes" or "no" based on the criteria below.

First, list all explicit user requests made throughout the conversation in the rationale section.
Second, for each request, determine whether it was addressed by the assistant by the end of the conversation,\
and **quote** the assistant's explicit response in the rationale section if you judge the request as addressed.
If there is no explicit response to a request—or the response can only be inferred from context—mark that request as incomplete.
Requests may be satisfied at any point in the dialogue as long as they are resolved by the final turn.
A refusal counts as addressed only if the assistant provides a clear and explicit explanation; refusals without reasoning should be marked incomplete.
Do not assume completeness merely because the user seems satisfied; evaluate solely whether each identified request was actually fulfilled.
Output "no" only if one or more user requests remain unaddressed in the final state. Output "yes" if all requests were addressed.
Base your judgment strictly on information explicitly stated or strongly implied in the conversation, without using outside assumptions.

<conversation>{{ conversation }}</conversation>
"""  # noqa: E501



from abc import ABC, abstractmethod
from typing import Any, Iterator

from mlflow.entities._job import Job
from mlflow.entities._job_status import JobStatus
from mlflow.utils.annotations import developer_stable


@developer_stable
class AbstractJobStore(ABC):
    """
    Abstract class that defines API interfaces for storing Job metadata.
    """

    @abstractmethod
    def create_job(self, function_fullname: str, params: str, timeout: float | None = None) -> Job:
        """
        Create a new job with the specified function and parameters.

        Args:
            function_fullname: The full name of the function to execute
            params: The job parameters that are serialized as a JSON string
            timeout: The job execution timeout in seconds

        Returns:
            Job entity instance
        """

    @abstractmethod
    def start_job(self, job_id: str) -> None:
        """
        Start a job by setting its status to RUNNING.

        Args:
            job_id: The ID of the job to start
        """

    @abstractmethod
    def reset_job(self, job_id: str) -> None:
        """
        Reset a job by setting its status to PENDING.

        Args:
            job_id: The ID of the job to re-enqueue.
        """

    @abstractmethod
    def finish_job(self, job_id: str, result: str) -> None:
        """
        Finish a job by setting its status to DONE and setting the result.

        Args:
            job_id: The ID of the job to finish
            result: The job result as a string
        """

    @abstractmethod
    def mark_job_timed_out(self, job_id: str) -> None:
        """
        Set a job status to Timeout.

        Args:
            job_id: The ID of the job
        """

    @abstractmethod
    def fail_job(self, job_id: str, error: str) -> None:
        """
        Fail a job by setting its status to FAILED and setting the error message.

        Args:
            job_id: The ID of the job to fail
            error: The error message as a string
        """

    @abstractmethod
    def retry_or_fail_job(self, job_id: str, error: str) -> int | None:
        """
        If the job retry_count is less than maximum allowed retry count,
        increment the retry_count and reset the job to PENDING status,
        otherwise set the job to FAILED status and fill the job's error field.

        Args:
            job_id: The ID of the job to fail
            error: The error message as a string

        Returns:
            If the job is allowed to retry, returns the retry count,
            otherwise returns None.
        """

    @abstractmethod
    def list_jobs(
        self,
        function_fullname: str | None = None,
        statuses: list[JobStatus] | None = None,
        begin_timestamp: int | None = None,
        end_timestamp: int | None = None,
        params: dict[str, Any] | None = None,
    ) -> Iterator[Job]:
        """
        List jobs based on the provided filters.

        Args:
            function_fullname: Filter by function full name (exact match)
            statuses: Filter by a list of job status (PENDING, RUNNING, DONE, FAILED, TIMEOUT)
            begin_timestamp: Filter jobs created after this timestamp (inclusive)
            end_timestamp: Filter jobs created before this timestamp (inclusive)
            params: Filter jobs by matching job params dict with the provided params dict
                e.g., if `params` is ``{'a': 3, 'b': 4}``, it can match the following job params:
                ``{'a': 3, 'b': 4}``, ``{'a': 3, 'b': 4, 'c': 5}``, but it does not match the
                following job params: ``{'a': 3, 'b': 6}``, ``{'a': 3, 'c': 5}``.

        Returns:
            Iterator of Job entities that match the filters, ordered by creation time (oldest first)
        """

    @abstractmethod
    def get_job(self, job_id: str) -> Job:
        """
        Get a job by its ID.

        Args:
            job_id: The ID of the job to retrieve

        Returns:
            Job entity

        Raises:
            MlflowException: If job with the given ID is not found
        """
